# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

from unittest.mock import ANY

from databuilder.serializers.neptune_serializer import (
    METADATA_KEY_PROPERTY_NAME, NEPTUNE_CREATION_TYPE_JOB, NEPTUNE_CREATION_TYPE_NODE_PROPERTY_NAME_BULK_LOADER_FORMAT,
    NEPTUNE_CREATION_TYPE_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT, NEPTUNE_HEADER_ID, NEPTUNE_HEADER_LABEL,
    NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT, NEPTUNE_RELATIONSHIP_HEADER_FROM,
    NEPTUNE_RELATIONSHIP_HEADER_TO,
)

EXPECTED_NEPTUNE_NODES = [
    {
        'name:String(single)': 'test_table1',
        NEPTUNE_HEADER_ID: 'Table:hive://gold.test_schema1/test_table1',
        METADATA_KEY_PROPERTY_NAME: 'Table:hive://gold.test_schema1/test_table1',
        NEPTUNE_HEADER_LABEL: 'Table',
        NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
        NEPTUNE_CREATION_TYPE_NODE_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB,
        'is_view:Bool(single)': False
    },
    {
        'description:String(single)': 'test_table1',
        NEPTUNE_HEADER_ID: 'Description:hive://gold.test_schema1/test_table1/_description',
        METADATA_KEY_PROPERTY_NAME: 'Description:hive://gold.test_schema1/test_table1/_description',
        NEPTUNE_HEADER_LABEL: 'Description',
        NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
        NEPTUNE_CREATION_TYPE_NODE_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB,
        'description_source:String(single)': 'description'
    },
    {
        'sort_order:Long(single)': 0,
        'col_type:String(single)': 'bigint',
        'name:String(single)': 'test_id1',
        NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
        NEPTUNE_CREATION_TYPE_NODE_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB,
        NEPTUNE_HEADER_ID: 'Column:hive://gold.test_schema1/test_table1/test_id1',
        METADATA_KEY_PROPERTY_NAME: 'Column:hive://gold.test_schema1/test_table1/test_id1',
        NEPTUNE_HEADER_LABEL: 'Column'
    },
    {
        'description:String(single)': 'description of test_table1',
        NEPTUNE_HEADER_ID: 'Description:hive://gold.test_schema1/test_table1/test_id1/_description',
        METADATA_KEY_PROPERTY_NAME: 'Description:hive://gold.test_schema1/test_table1/test_id1/_description',
        NEPTUNE_HEADER_LABEL: 'Description',
        NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
        NEPTUNE_CREATION_TYPE_NODE_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB,
        'description_source:String(single)': 'description'
    },
    {
        'sort_order:Long(single)': 1,
        'col_type:String(single)': 'bigint',
        'name:String(single)': 'test_id2',
        NEPTUNE_HEADER_ID: 'Column:hive://gold.test_schema1/test_table1/test_id2',
        METADATA_KEY_PROPERTY_NAME: 'Column:hive://gold.test_schema1/test_table1/test_id2',
        NEPTUNE_HEADER_LABEL: 'Column',
        NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
        NEPTUNE_CREATION_TYPE_NODE_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB,
    },
    {
        'description:String(single)': 'description of test_id2',
        NEPTUNE_HEADER_ID: 'Description:hive://gold.test_schema1/test_table1/test_id2/_description',
        METADATA_KEY_PROPERTY_NAME: 'Description:hive://gold.test_schema1/test_table1/test_id2/_description',
        NEPTUNE_HEADER_LABEL: 'Description',
        NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
        NEPTUNE_CREATION_TYPE_NODE_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB,
        'description_source:String(single)': 'description'
    },
    {
        'sort_order:Long(single)': 2,
        'col_type:String(single)': 'boolean',
        'name:String(single)': 'is_active',
        NEPTUNE_HEADER_ID: 'Column:hive://gold.test_schema1/test_table1/is_active',
        METADATA_KEY_PROPERTY_NAME: 'Column:hive://gold.test_schema1/test_table1/is_active',
        NEPTUNE_HEADER_LABEL: 'Column',
        NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
        NEPTUNE_CREATION_TYPE_NODE_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB,
    },
    {
        'sort_order:Long(single)': 3,
        'col_type:String(single)': 'varchar',
        'name:String(single)': 'source',
        NEPTUNE_HEADER_ID: 'Column:hive://gold.test_schema1/test_table1/source',
        METADATA_KEY_PROPERTY_NAME: 'Column:hive://gold.test_schema1/test_table1/source',
        NEPTUNE_HEADER_LABEL: 'Column',
        NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
        NEPTUNE_CREATION_TYPE_NODE_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB,
    },
    {
        'description:String(single)': 'description of source',
        NEPTUNE_HEADER_ID: 'Description:hive://gold.test_schema1/test_table1/source/_description',
        METADATA_KEY_PROPERTY_NAME: 'Description:hive://gold.test_schema1/test_table1/source/_description',
        NEPTUNE_HEADER_LABEL: 'Description',
        NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
        NEPTUNE_CREATION_TYPE_NODE_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB,
        'description_source:String(single)': 'description'
    },
    {
        'sort_order:Long(single)': 4,
        'col_type:String(single)': 'timestamp',
        'name:String(single)': 'etl_created_at',
        NEPTUNE_HEADER_ID: 'Column:hive://gold.test_schema1/test_table1/etl_created_at',
        METADATA_KEY_PROPERTY_NAME: 'Column:hive://gold.test_schema1/test_table1/etl_created_at',
        NEPTUNE_HEADER_LABEL: 'Column',
        NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
        NEPTUNE_CREATION_TYPE_NODE_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB,
    },
    {
        'description:String(single)': 'description of etl_created_at',
        NEPTUNE_HEADER_ID: 'Description:hive://gold.test_schema1/test_table1/etl_created_at/_description',
        METADATA_KEY_PROPERTY_NAME: 'Description:hive://gold.test_schema1/test_table1/etl_created_at/_description',
        NEPTUNE_HEADER_LABEL: 'Description',
        NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
        NEPTUNE_CREATION_TYPE_NODE_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB,
        'description_source:String(single)': 'description'
    },
    {
        'sort_order:Long(single)': 5,
        'col_type:String(single)': 'varchar',
        'name:String(single)': 'ds',
        NEPTUNE_HEADER_ID: 'Column:hive://gold.test_schema1/test_table1/ds',
        METADATA_KEY_PROPERTY_NAME: 'Column:hive://gold.test_schema1/test_table1/ds',
        NEPTUNE_HEADER_LABEL: 'Column',
        NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
        NEPTUNE_CREATION_TYPE_NODE_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB,
    },
    {
        'name:String(single)': 'hive',
        NEPTUNE_HEADER_ID: 'Database:database://hive',
        METADATA_KEY_PROPERTY_NAME: 'Database:database://hive',
        NEPTUNE_HEADER_LABEL: 'Database',
        NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
        NEPTUNE_CREATION_TYPE_NODE_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB,
    },
    {
        'name:String(single)': 'gold',
        NEPTUNE_HEADER_ID: 'Cluster:hive://gold',
        METADATA_KEY_PROPERTY_NAME: 'Cluster:hive://gold',
        NEPTUNE_HEADER_LABEL: 'Cluster',
        NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
        NEPTUNE_CREATION_TYPE_NODE_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB,
    },
    {
        'name:String(single)': 'test_schema1',
        NEPTUNE_HEADER_ID: 'Schema:hive://gold.test_schema1',
        METADATA_KEY_PROPERTY_NAME: 'Schema:hive://gold.test_schema1',
        NEPTUNE_HEADER_LABEL: 'Schema',
        NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
        NEPTUNE_CREATION_TYPE_NODE_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB,
    }
]

EXPECTED_RELATIONSHIPS_NEPTUNE = [
    [
        {
            NEPTUNE_HEADER_ID: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Schema:hive://gold.test_schema1',
                to_vertex_id='Table:hive://gold.test_schema1/test_table1',
                label='TABLE'
            ),
            METADATA_KEY_PROPERTY_NAME: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Schema:hive://gold.test_schema1',
                to_vertex_id='Table:hive://gold.test_schema1/test_table1',
                label='TABLE'
            ),
            NEPTUNE_RELATIONSHIP_HEADER_FROM: 'Schema:hive://gold.test_schema1',
            NEPTUNE_RELATIONSHIP_HEADER_TO: 'Table:hive://gold.test_schema1/test_table1',
            NEPTUNE_HEADER_LABEL: 'TABLE',
            NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
            NEPTUNE_CREATION_TYPE_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB
        },
        {
            NEPTUNE_HEADER_ID: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Table:hive://gold.test_schema1/test_table1',
                to_vertex_id='Schema:hive://gold.test_schema1',
                label='TABLE_OF'
            ),
            METADATA_KEY_PROPERTY_NAME: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Table:hive://gold.test_schema1/test_table1',
                to_vertex_id='Schema:hive://gold.test_schema1',
                label='TABLE_OF'
            ),
            NEPTUNE_RELATIONSHIP_HEADER_FROM: 'Table:hive://gold.test_schema1/test_table1',
            NEPTUNE_RELATIONSHIP_HEADER_TO: 'Schema:hive://gold.test_schema1',
            NEPTUNE_HEADER_LABEL: 'TABLE_OF',
            NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
            NEPTUNE_CREATION_TYPE_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB
        }
    ],
    [
        {
            NEPTUNE_HEADER_ID: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Table:hive://gold.test_schema1/test_table1',
                to_vertex_id='Description:hive://gold.test_schema1/test_table1/_description',
                label='DESCRIPTION'
            ),
            METADATA_KEY_PROPERTY_NAME: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Table:hive://gold.test_schema1/test_table1',
                to_vertex_id='Description:hive://gold.test_schema1/test_table1/_description',
                label='DESCRIPTION'
            ),
            NEPTUNE_RELATIONSHIP_HEADER_FROM: 'Table:hive://gold.test_schema1/test_table1',
            NEPTUNE_RELATIONSHIP_HEADER_TO: 'Description:hive://gold.test_schema1/test_table1/_description',
            NEPTUNE_HEADER_LABEL: 'DESCRIPTION',
            NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
            NEPTUNE_CREATION_TYPE_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB
        },
        {
            NEPTUNE_HEADER_ID: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Description:hive://gold.test_schema1/test_table1/_description',
                to_vertex_id='Table:hive://gold.test_schema1/test_table1',
                label='DESCRIPTION_OF'
            ),
            METADATA_KEY_PROPERTY_NAME: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Description:hive://gold.test_schema1/test_table1/_description',
                to_vertex_id='Table:hive://gold.test_schema1/test_table1',
                label='DESCRIPTION_OF'
            ),
            NEPTUNE_RELATIONSHIP_HEADER_FROM: 'Description:hive://gold.test_schema1/test_table1/_description',
            NEPTUNE_RELATIONSHIP_HEADER_TO: 'Table:hive://gold.test_schema1/test_table1',
            NEPTUNE_HEADER_LABEL: 'DESCRIPTION_OF',
            NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
            NEPTUNE_CREATION_TYPE_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB
        }
    ],
    [
        {
            NEPTUNE_HEADER_ID: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Table:hive://gold.test_schema1/test_table1',
                to_vertex_id='Column:hive://gold.test_schema1/test_table1/test_id1',
                label='COLUMN'
            ),
            METADATA_KEY_PROPERTY_NAME: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Table:hive://gold.test_schema1/test_table1',
                to_vertex_id='Column:hive://gold.test_schema1/test_table1/test_id1',
                label='COLUMN'
            ),
            NEPTUNE_RELATIONSHIP_HEADER_FROM: 'Table:hive://gold.test_schema1/test_table1',
            NEPTUNE_RELATIONSHIP_HEADER_TO: 'Column:hive://gold.test_schema1/test_table1/test_id1',
            NEPTUNE_HEADER_LABEL: 'COLUMN',
            NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
            NEPTUNE_CREATION_TYPE_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB
        },
        {
            NEPTUNE_HEADER_ID: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Column:hive://gold.test_schema1/test_table1/test_id1',
                to_vertex_id='Table:hive://gold.test_schema1/test_table1',
                label='COLUMN_OF'
            ),
            METADATA_KEY_PROPERTY_NAME: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Column:hive://gold.test_schema1/test_table1/test_id1',
                to_vertex_id='Table:hive://gold.test_schema1/test_table1',
                label='COLUMN_OF'
            ),
            NEPTUNE_RELATIONSHIP_HEADER_FROM: 'Column:hive://gold.test_schema1/test_table1/test_id1',
            NEPTUNE_RELATIONSHIP_HEADER_TO: 'Table:hive://gold.test_schema1/test_table1',
            NEPTUNE_HEADER_LABEL: 'COLUMN_OF',
            NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
            NEPTUNE_CREATION_TYPE_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB
        }
    ],
    [
        {
            NEPTUNE_HEADER_ID: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Column:hive://gold.test_schema1/test_table1/test_id1',
                to_vertex_id='Description:hive://gold.test_schema1/test_table1/test_id1/_description',
                label='DESCRIPTION'
            ),
            METADATA_KEY_PROPERTY_NAME: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Column:hive://gold.test_schema1/test_table1/test_id1',
                to_vertex_id='Description:hive://gold.test_schema1/test_table1/test_id1/_description',
                label='DESCRIPTION'
            ),
            NEPTUNE_RELATIONSHIP_HEADER_FROM: 'Column:hive://gold.test_schema1/test_table1/test_id1',
            NEPTUNE_RELATIONSHIP_HEADER_TO: 'Description:hive://gold.test_schema1/test_table1/test_id1/_description',
            NEPTUNE_HEADER_LABEL: 'DESCRIPTION',
            NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
            NEPTUNE_CREATION_TYPE_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB
        },
        {
            NEPTUNE_HEADER_ID: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Description:hive://gold.test_schema1/test_table1/test_id1/_description',
                to_vertex_id='Column:hive://gold.test_schema1/test_table1/test_id1',
                label='DESCRIPTION_OF'
            ),
            METADATA_KEY_PROPERTY_NAME: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Description:hive://gold.test_schema1/test_table1/test_id1/_description',
                to_vertex_id='Column:hive://gold.test_schema1/test_table1/test_id1',
                label='DESCRIPTION_OF'
            ),
            NEPTUNE_RELATIONSHIP_HEADER_FROM: 'Description:hive://gold.test_schema1/test_table1/test_id1/_description',
            NEPTUNE_RELATIONSHIP_HEADER_TO: 'Column:hive://gold.test_schema1/test_table1/test_id1',
            NEPTUNE_HEADER_LABEL: 'DESCRIPTION_OF',
            NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
            NEPTUNE_CREATION_TYPE_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB
        }
    ],
    [
        {
            NEPTUNE_HEADER_ID: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Table:hive://gold.test_schema1/test_table1',
                to_vertex_id='Column:hive://gold.test_schema1/test_table1/test_id2',
                label='COLUMN'
            ),
            METADATA_KEY_PROPERTY_NAME: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Table:hive://gold.test_schema1/test_table1',
                to_vertex_id='Column:hive://gold.test_schema1/test_table1/test_id2',
                label='COLUMN'
            ),
            NEPTUNE_RELATIONSHIP_HEADER_FROM: 'Table:hive://gold.test_schema1/test_table1',
            NEPTUNE_RELATIONSHIP_HEADER_TO: 'Column:hive://gold.test_schema1/test_table1/test_id2',
            NEPTUNE_HEADER_LABEL: 'COLUMN',
            NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
            NEPTUNE_CREATION_TYPE_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB
        },
        {
            NEPTUNE_HEADER_ID: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Column:hive://gold.test_schema1/test_table1/test_id2',
                to_vertex_id='Table:hive://gold.test_schema1/test_table1',
                label='COLUMN_OF'
            ),
            METADATA_KEY_PROPERTY_NAME: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Column:hive://gold.test_schema1/test_table1/test_id2',
                to_vertex_id='Table:hive://gold.test_schema1/test_table1',
                label='COLUMN_OF'
            ),
            NEPTUNE_RELATIONSHIP_HEADER_FROM: 'Column:hive://gold.test_schema1/test_table1/test_id2',
            NEPTUNE_RELATIONSHIP_HEADER_TO: 'Table:hive://gold.test_schema1/test_table1',
            NEPTUNE_HEADER_LABEL: 'COLUMN_OF',
            NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
            NEPTUNE_CREATION_TYPE_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB
        }
    ],
    [
        {
            NEPTUNE_HEADER_ID: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Column:hive://gold.test_schema1/test_table1/test_id2',
                to_vertex_id='Description:hive://gold.test_schema1/test_table1/test_id2/_description',
                label='DESCRIPTION'
            ),
            METADATA_KEY_PROPERTY_NAME: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Column:hive://gold.test_schema1/test_table1/test_id2',
                to_vertex_id='Description:hive://gold.test_schema1/test_table1/test_id2/_description',
                label='DESCRIPTION'
            ),
            NEPTUNE_RELATIONSHIP_HEADER_FROM: 'Column:hive://gold.test_schema1/test_table1/test_id2',
            NEPTUNE_RELATIONSHIP_HEADER_TO: 'Description:hive://gold.test_schema1/test_table1/test_id2/_description',
            NEPTUNE_HEADER_LABEL: 'DESCRIPTION',
            NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
            NEPTUNE_CREATION_TYPE_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB
        },
        {
            NEPTUNE_HEADER_ID: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Description:hive://gold.test_schema1/test_table1/test_id2/_description',
                to_vertex_id='Column:hive://gold.test_schema1/test_table1/test_id2',
                label='DESCRIPTION_OF'
            ),
            METADATA_KEY_PROPERTY_NAME: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Description:hive://gold.test_schema1/test_table1/test_id2/_description',
                to_vertex_id='Column:hive://gold.test_schema1/test_table1/test_id2',
                label='DESCRIPTION_OF'
            ),
            NEPTUNE_RELATIONSHIP_HEADER_FROM: 'Description:hive://gold.test_schema1/test_table1/test_id2/_description',
            NEPTUNE_RELATIONSHIP_HEADER_TO: 'Column:hive://gold.test_schema1/test_table1/test_id2',
            NEPTUNE_HEADER_LABEL: 'DESCRIPTION_OF',
            NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
            NEPTUNE_CREATION_TYPE_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB
        }
    ],
    [
        {
            NEPTUNE_HEADER_ID: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Table:hive://gold.test_schema1/test_table1',
                to_vertex_id='Column:hive://gold.test_schema1/test_table1/is_active',
                label='COLUMN'
            ),
            METADATA_KEY_PROPERTY_NAME: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Table:hive://gold.test_schema1/test_table1',
                to_vertex_id='Column:hive://gold.test_schema1/test_table1/is_active',
                label='COLUMN'
            ),
            NEPTUNE_RELATIONSHIP_HEADER_FROM: 'Table:hive://gold.test_schema1/test_table1',
            NEPTUNE_RELATIONSHIP_HEADER_TO: 'Column:hive://gold.test_schema1/test_table1/is_active',
            NEPTUNE_HEADER_LABEL: 'COLUMN',
            NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
            NEPTUNE_CREATION_TYPE_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB
        },
        {
            NEPTUNE_HEADER_ID: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Column:hive://gold.test_schema1/test_table1/is_active',
                to_vertex_id='Table:hive://gold.test_schema1/test_table1',
                label='COLUMN_OF'
            ),
            METADATA_KEY_PROPERTY_NAME: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Column:hive://gold.test_schema1/test_table1/is_active',
                to_vertex_id='Table:hive://gold.test_schema1/test_table1',
                label='COLUMN_OF'
            ),
            NEPTUNE_RELATIONSHIP_HEADER_FROM: 'Column:hive://gold.test_schema1/test_table1/is_active',
            NEPTUNE_RELATIONSHIP_HEADER_TO: 'Table:hive://gold.test_schema1/test_table1',
            NEPTUNE_HEADER_LABEL: 'COLUMN_OF',
            NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
            NEPTUNE_CREATION_TYPE_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB
        }
    ],
    [
        {
            NEPTUNE_HEADER_ID: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Table:hive://gold.test_schema1/test_table1',
                to_vertex_id='Column:hive://gold.test_schema1/test_table1/source',
                label='COLUMN'
            ),
            METADATA_KEY_PROPERTY_NAME: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Table:hive://gold.test_schema1/test_table1',
                to_vertex_id='Column:hive://gold.test_schema1/test_table1/source',
                label='COLUMN'
            ),
            NEPTUNE_RELATIONSHIP_HEADER_FROM: 'Table:hive://gold.test_schema1/test_table1',
            NEPTUNE_RELATIONSHIP_HEADER_TO: 'Column:hive://gold.test_schema1/test_table1/source',
            NEPTUNE_HEADER_LABEL: 'COLUMN',
            NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
            NEPTUNE_CREATION_TYPE_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB
        },
        {
            NEPTUNE_HEADER_ID: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Column:hive://gold.test_schema1/test_table1/source',
                to_vertex_id='Table:hive://gold.test_schema1/test_table1',
                label='COLUMN_OF'
            ),
            METADATA_KEY_PROPERTY_NAME: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Column:hive://gold.test_schema1/test_table1/source',
                to_vertex_id='Table:hive://gold.test_schema1/test_table1',
                label='COLUMN_OF'
            ),
            NEPTUNE_RELATIONSHIP_HEADER_FROM: 'Column:hive://gold.test_schema1/test_table1/source',
            NEPTUNE_RELATIONSHIP_HEADER_TO: 'Table:hive://gold.test_schema1/test_table1',
            NEPTUNE_HEADER_LABEL: 'COLUMN_OF',
            NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
            NEPTUNE_CREATION_TYPE_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB
        }
    ],
    [
        {
            NEPTUNE_HEADER_ID: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Column:hive://gold.test_schema1/test_table1/source',
                to_vertex_id='Description:hive://gold.test_schema1/test_table1/source/_description',
                label='DESCRIPTION'
            ),
            METADATA_KEY_PROPERTY_NAME: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Column:hive://gold.test_schema1/test_table1/source',
                to_vertex_id='Description:hive://gold.test_schema1/test_table1/source/_description',
                label='DESCRIPTION'
            ),
            NEPTUNE_RELATIONSHIP_HEADER_FROM: 'Column:hive://gold.test_schema1/test_table1/source',
            NEPTUNE_RELATIONSHIP_HEADER_TO: 'Description:hive://gold.test_schema1/test_table1/source/_description',
            NEPTUNE_HEADER_LABEL: 'DESCRIPTION',
            NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
            NEPTUNE_CREATION_TYPE_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB
        },
        {
            NEPTUNE_HEADER_ID: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Description:hive://gold.test_schema1/test_table1/source/_description',
                to_vertex_id='Column:hive://gold.test_schema1/test_table1/source',
                label='DESCRIPTION_OF'
            ),
            METADATA_KEY_PROPERTY_NAME: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Description:hive://gold.test_schema1/test_table1/source/_description',
                to_vertex_id='Column:hive://gold.test_schema1/test_table1/source',
                label='DESCRIPTION_OF'
            ),
            NEPTUNE_RELATIONSHIP_HEADER_FROM: 'Description:hive://gold.test_schema1/test_table1/source/_description',
            NEPTUNE_RELATIONSHIP_HEADER_TO: 'Column:hive://gold.test_schema1/test_table1/source',
            NEPTUNE_HEADER_LABEL: 'DESCRIPTION_OF',
            NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
            NEPTUNE_CREATION_TYPE_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB
        }
    ],
    [
        {
            NEPTUNE_HEADER_ID: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Table:hive://gold.test_schema1/test_table1',
                to_vertex_id='Column:hive://gold.test_schema1/test_table1/etl_created_at',
                label='COLUMN'
            ),
            METADATA_KEY_PROPERTY_NAME: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Table:hive://gold.test_schema1/test_table1',
                to_vertex_id='Column:hive://gold.test_schema1/test_table1/etl_created_at',
                label='COLUMN'
            ),
            NEPTUNE_RELATIONSHIP_HEADER_FROM: 'Table:hive://gold.test_schema1/test_table1',
            NEPTUNE_RELATIONSHIP_HEADER_TO: 'Column:hive://gold.test_schema1/test_table1/etl_created_at',
            NEPTUNE_HEADER_LABEL: 'COLUMN',
            NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
            NEPTUNE_CREATION_TYPE_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB
        },
        {
            NEPTUNE_HEADER_ID: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Column:hive://gold.test_schema1/test_table1/etl_created_at',
                to_vertex_id='Table:hive://gold.test_schema1/test_table1',
                label='COLUMN_OF'
            ),
            METADATA_KEY_PROPERTY_NAME: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Column:hive://gold.test_schema1/test_table1/etl_created_at',
                to_vertex_id='Table:hive://gold.test_schema1/test_table1',
                label='COLUMN_OF'
            ),
            NEPTUNE_RELATIONSHIP_HEADER_FROM: 'Column:hive://gold.test_schema1/test_table1/etl_created_at',
            NEPTUNE_RELATIONSHIP_HEADER_TO: 'Table:hive://gold.test_schema1/test_table1',
            NEPTUNE_HEADER_LABEL: 'COLUMN_OF',
            NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
            NEPTUNE_CREATION_TYPE_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB
        }
    ],
    [
        {
            NEPTUNE_HEADER_ID: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Column:hive://gold.test_schema1/test_table1/etl_created_at',
                to_vertex_id='Description:hive://gold.test_schema1/test_table1/etl_created_at/_description',
                label='DESCRIPTION'
            ),
            METADATA_KEY_PROPERTY_NAME: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Column:hive://gold.test_schema1/test_table1/etl_created_at',
                to_vertex_id='Description:hive://gold.test_schema1/test_table1/etl_created_at/_description',
                label='DESCRIPTION'
            ),
            NEPTUNE_RELATIONSHIP_HEADER_FROM: 'Column:hive://gold.test_schema1/test_table1/etl_created_at',
            NEPTUNE_RELATIONSHIP_HEADER_TO:
                'Description:hive://gold.test_schema1/test_table1/etl_created_at/_description',
            NEPTUNE_HEADER_LABEL: 'DESCRIPTION',
            NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
            NEPTUNE_CREATION_TYPE_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB
        },
        {
            NEPTUNE_HEADER_ID: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Description:hive://gold.test_schema1/test_table1/etl_created_at/_description',
                to_vertex_id='Column:hive://gold.test_schema1/test_table1/etl_created_at',
                label='DESCRIPTION_OF'
            ),
            METADATA_KEY_PROPERTY_NAME: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Description:hive://gold.test_schema1/test_table1/etl_created_at/_description',
                to_vertex_id='Column:hive://gold.test_schema1/test_table1/etl_created_at',
                label='DESCRIPTION_OF'
            ),
            NEPTUNE_RELATIONSHIP_HEADER_FROM:
                'Description:hive://gold.test_schema1/test_table1/etl_created_at/_description',
            NEPTUNE_RELATIONSHIP_HEADER_TO: 'Column:hive://gold.test_schema1/test_table1/etl_created_at',
            NEPTUNE_HEADER_LABEL: 'DESCRIPTION_OF',
            NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
            NEPTUNE_CREATION_TYPE_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB
        }
    ],
    [
        {
            NEPTUNE_HEADER_ID: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Table:hive://gold.test_schema1/test_table1',
                to_vertex_id='Column:hive://gold.test_schema1/test_table1/ds',
                label='COLUMN'
            ),
            METADATA_KEY_PROPERTY_NAME: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Table:hive://gold.test_schema1/test_table1',
                to_vertex_id='Column:hive://gold.test_schema1/test_table1/ds',
                label='COLUMN'
            ),
            NEPTUNE_RELATIONSHIP_HEADER_FROM: 'Table:hive://gold.test_schema1/test_table1',
            NEPTUNE_RELATIONSHIP_HEADER_TO: 'Column:hive://gold.test_schema1/test_table1/ds',
            NEPTUNE_HEADER_LABEL: 'COLUMN',
            NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
            NEPTUNE_CREATION_TYPE_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB
        },
        {
            NEPTUNE_HEADER_ID: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Column:hive://gold.test_schema1/test_table1/ds',
                to_vertex_id='Table:hive://gold.test_schema1/test_table1',
                label='COLUMN_OF'
            ),
            METADATA_KEY_PROPERTY_NAME: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Column:hive://gold.test_schema1/test_table1/ds',
                to_vertex_id='Table:hive://gold.test_schema1/test_table1',
                label='COLUMN_OF'
            ),
            NEPTUNE_RELATIONSHIP_HEADER_FROM: 'Column:hive://gold.test_schema1/test_table1/ds',
            NEPTUNE_RELATIONSHIP_HEADER_TO: 'Table:hive://gold.test_schema1/test_table1',
            NEPTUNE_HEADER_LABEL: 'COLUMN_OF',
            NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
            NEPTUNE_CREATION_TYPE_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB
        }
    ],
    [
        {
            NEPTUNE_CREATION_TYPE_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB,
            NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
            NEPTUNE_RELATIONSHIP_HEADER_FROM: 'Database:database://hive',
            NEPTUNE_HEADER_ID: 'CLUSTER:Database:database://hive_Cluster:hive://gold',
            METADATA_KEY_PROPERTY_NAME: 'CLUSTER:Database:database://hive_Cluster:hive://gold',
            NEPTUNE_HEADER_LABEL: 'CLUSTER',
            NEPTUNE_RELATIONSHIP_HEADER_TO: 'Cluster:hive://gold'
        },
        {
            NEPTUNE_CREATION_TYPE_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB,
            NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
            NEPTUNE_RELATIONSHIP_HEADER_FROM: 'Cluster:hive://gold',
            NEPTUNE_HEADER_ID: 'CLUSTER_OF:Cluster:hive://gold_Database:database://hive',
            METADATA_KEY_PROPERTY_NAME: 'CLUSTER_OF:Cluster:hive://gold_Database:database://hive',
            NEPTUNE_HEADER_LABEL: 'CLUSTER_OF',
            NEPTUNE_RELATIONSHIP_HEADER_TO: 'Database:database://hive'
        }
    ],
    [
        {
            NEPTUNE_CREATION_TYPE_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB,
            NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
            NEPTUNE_RELATIONSHIP_HEADER_FROM: 'Cluster:hive://gold',
            NEPTUNE_HEADER_ID: 'SCHEMA:Cluster:hive://gold_Schema:hive://gold.test_schema1',
            METADATA_KEY_PROPERTY_NAME: 'SCHEMA:Cluster:hive://gold_Schema:hive://gold.test_schema1',
            NEPTUNE_HEADER_LABEL: 'SCHEMA',
            NEPTUNE_RELATIONSHIP_HEADER_TO: 'Schema:hive://gold.test_schema1'
        },
        {
            NEPTUNE_CREATION_TYPE_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB,
            NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
            NEPTUNE_RELATIONSHIP_HEADER_FROM: 'Schema:hive://gold.test_schema1',
            NEPTUNE_HEADER_ID: 'SCHEMA_OF:Schema:hive://gold.test_schema1_Cluster:hive://gold',
            METADATA_KEY_PROPERTY_NAME: 'SCHEMA_OF:Schema:hive://gold.test_schema1_Cluster:hive://gold',
            NEPTUNE_HEADER_LABEL: 'SCHEMA_OF',
            NEPTUNE_RELATIONSHIP_HEADER_TO: 'Cluster:hive://gold'
        }
    ]
]

EXPECTED_RECORDS_MYSQL = [
    {
        'rk': 'database://hive',
        'name': 'hive'
    },
    {
        'rk': 'hive://gold',
        'name': 'gold',
        'database_rk': 'database://hive'
    },
    {
        'rk': 'hive://gold.test_schema1',
        'name': 'test_schema1',
        'cluster_rk': 'hive://gold',
    },
    {
        'rk': 'hive://gold.test_schema1/test_table1',
        'name': 'test_table1',
        'is_view': False,
        'schema_rk': 'hive://gold.test_schema1'
    },
    {
        'rk': 'hive://gold.test_schema1/test_table1/_description',
        'description': 'test_table1',
        'description_source': 'description',
        'table_rk': 'hive://gold.test_schema1/test_table1'
    },
    {
        'rk': 'hive://gold.test_schema1/test_table1/test_id1',
        'name': 'test_id1',
        'type': 'bigint',
        'sort_order': 0,
        'table_rk': 'hive://gold.test_schema1/test_table1'
    },
    {
        'rk': 'hive://gold.test_schema1/test_table1/test_id1/_description',
        'description': 'description of test_table1',
        'description_source': 'description',
        'column_rk': 'hive://gold.test_schema1/test_table1/test_id1'
    },
    {
        'rk': 'hive://gold.test_schema1/test_table1/test_id2',
        'name': 'test_id2',
        'type': 'bigint',
        'sort_order': 1,
        'table_rk': 'hive://gold.test_schema1/test_table1'
    },
    {
        'rk': 'hive://gold.test_schema1/test_table1/test_id2/_description',
        'description': 'description of test_id2',
        'description_source': 'description',
        'column_rk': 'hive://gold.test_schema1/test_table1/test_id2'
    },
    {
        'rk': 'hive://gold.test_schema1/test_table1/is_active',
        'name': 'is_active',
        'type': 'boolean',
        'sort_order': 2,
        'table_rk': 'hive://gold.test_schema1/test_table1'
    },
    {
        'rk': 'hive://gold.test_schema1/test_table1/source',
        'name': 'source',
        'type': 'varchar',
        'sort_order': 3,
        'table_rk': 'hive://gold.test_schema1/test_table1'
    },
    {
        'rk': 'hive://gold.test_schema1/test_table1/source/_description',
        'description': 'description of source',
        'description_source': 'description',
        'column_rk': 'hive://gold.test_schema1/test_table1/source'
    },
    {
        'rk': 'hive://gold.test_schema1/test_table1/etl_created_at',
        'name': 'etl_created_at',
        'type': 'timestamp',
        'sort_order': 4,
        'table_rk': 'hive://gold.test_schema1/test_table1'
    },
    {
        'rk': 'hive://gold.test_schema1/test_table1/etl_created_at/_description',
        'description': 'description of etl_created_at',
        'description_source': 'description',
        'column_rk': 'hive://gold.test_schema1/test_table1/etl_created_at'
    },
    {
        'rk': 'hive://gold.test_schema1/test_table1/ds',
        'name': 'ds',
        'type': 'varchar',
        'sort_order': 5,
        'table_rk': 'hive://gold.test_schema1/test_table1'
    }
]



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import unittest

from mock import MagicMock
from pyhocon import ConfigFactory

from databuilder.transformer.base_transformer import ChainedTransformer


class TestChainedTransformer(unittest.TestCase):
    def test_init_not_called(self) -> None:

        mock_transformer1 = MagicMock()
        mock_transformer1.transform.return_value = "foo"
        mock_transformer2 = MagicMock()
        mock_transformer2.transform.return_value = "bar"

        chained_transformer = ChainedTransformer(
            transformers=[mock_transformer1, mock_transformer2]
        )

        config = ConfigFactory.from_dict({})
        chained_transformer.init(conf=config)

        next(chained_transformer.transform({"foo": "bar"}))

        mock_transformer1.init.assert_not_called()
        mock_transformer1.transform.assert_called_once()
        mock_transformer2.init.assert_not_called()
        mock_transformer2.transform.assert_called_once()

    def test_init_called(self) -> None:

        mock_transformer1 = MagicMock()
        mock_transformer1.get_scope.return_value = "foo"
        mock_transformer1.transform.return_value = "foo"
        mock_transformer2 = MagicMock()
        mock_transformer2.get_scope.return_value = "bar"
        mock_transformer2.transform.return_value = "bar"

        chained_transformer = ChainedTransformer(
            transformers=[mock_transformer1, mock_transformer2],
            is_init_transformers=True,
        )

        config = ConfigFactory.from_dict({})
        chained_transformer.init(conf=config)

        next(chained_transformer.transform({"foo": "bar"}))

        mock_transformer1.init.assert_called_once()
        mock_transformer1.transform.assert_called_once()
        mock_transformer2.init.assert_called_once()
        mock_transformer2.transform.assert_called_once()

    def test_transformer_transforms(self) -> None:

        mock_transformer1 = MagicMock()
        mock_transformer1.transform.side_effect = lambda s: s + "b"
        mock_transformer2 = MagicMock()
        mock_transformer2.transform.side_effect = lambda s: s + "c"

        chained_transformer = ChainedTransformer(
            transformers=[mock_transformer1, mock_transformer2]
        )

        config = ConfigFactory.from_dict({})
        chained_transformer.init(conf=config)

        result = next(chained_transformer.transform("a"))
        self.assertEqual(result, "abc")



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import unittest
from unittest.mock import ANY

from databuilder.models.dashboard.dashboard_owner import DashboardOwner
from databuilder.models.graph_serializable import (
    RELATION_END_KEY, RELATION_END_LABEL, RELATION_REVERSE_TYPE, RELATION_START_KEY, RELATION_START_LABEL,
    RELATION_TYPE,
)
from databuilder.serializers import (
    mysql_serializer, neo4_serializer, neptune_serializer,
)
from databuilder.serializers.neptune_serializer import (
    METADATA_KEY_PROPERTY_NAME, NEPTUNE_CREATION_TYPE_JOB,
    NEPTUNE_CREATION_TYPE_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT, NEPTUNE_HEADER_ID, NEPTUNE_HEADER_LABEL,
    NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT, NEPTUNE_RELATIONSHIP_HEADER_FROM,
    NEPTUNE_RELATIONSHIP_HEADER_TO,
)


class TestDashboardOwner(unittest.TestCase):

    def setUp(self) -> None:
        self.dashboard_owner = DashboardOwner(
            email='foo@bar.com',
            cluster='cluster_id',
            product='product_id',
            dashboard_id='dashboard_id',
            dashboard_group_id='dashboard_group_id'
        )

    def test_dashboard_owner_nodes(self) -> None:
        actual = self.dashboard_owner.create_next_node()
        self.assertIsNone(actual)

    def test_dashboard_owner_relations(self) -> None:

        actual = self.dashboard_owner.create_next_relation()
        actual_serialized = neo4_serializer.serialize_relationship(actual)
        expected = {
            RELATION_END_KEY: 'foo@bar.com',
            RELATION_START_LABEL: 'Dashboard',
            RELATION_END_LABEL: 'User',
            RELATION_START_KEY: 'product_id_dashboard://cluster_id.dashboard_group_id/dashboard_id',
            RELATION_TYPE: 'OWNER',
            RELATION_REVERSE_TYPE: 'OWNER_OF'
        }
        assert actual is not None
        self.assertDictEqual(actual_serialized, expected)

    def test_dashboard_owner_relations_neptune(self) -> None:
        actual = self.dashboard_owner.create_next_relation()
        actual_serialized = neptune_serializer.convert_relationship(actual)
        neptune_forward_expected = {
            NEPTUNE_HEADER_ID: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Dashboard:product_id_dashboard://cluster_id.dashboard_group_id/dashboard_id',
                to_vertex_id='User:foo@bar.com',
                label='OWNER'
            ),
            METADATA_KEY_PROPERTY_NAME: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Dashboard:product_id_dashboard://cluster_id.dashboard_group_id/dashboard_id',
                to_vertex_id='User:foo@bar.com',
                label='OWNER'
            ),
            NEPTUNE_RELATIONSHIP_HEADER_FROM:
                'Dashboard:product_id_dashboard://cluster_id.dashboard_group_id/dashboard_id',
            NEPTUNE_RELATIONSHIP_HEADER_TO: 'User:foo@bar.com',
            NEPTUNE_HEADER_LABEL: 'OWNER',
            NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
            NEPTUNE_CREATION_TYPE_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB
        }

        neptune_reversed_expected = {
            NEPTUNE_HEADER_ID: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='User:foo@bar.com',
                to_vertex_id='Dashboard:product_id_dashboard://cluster_id.dashboard_group_id/dashboard_id',
                label='OWNER_OF'
            ),
            METADATA_KEY_PROPERTY_NAME: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='User:foo@bar.com',
                to_vertex_id='Dashboard:product_id_dashboard://cluster_id.dashboard_group_id/dashboard_id',
                label='OWNER_OF'
            ),
            NEPTUNE_RELATIONSHIP_HEADER_FROM: 'User:foo@bar.com',
            NEPTUNE_RELATIONSHIP_HEADER_TO:
                'Dashboard:product_id_dashboard://cluster_id.dashboard_group_id/dashboard_id',
            NEPTUNE_HEADER_LABEL: 'OWNER_OF',
            NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
            NEPTUNE_CREATION_TYPE_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB
        }

        assert actual is not None
        self.assertDictEqual(actual_serialized[0], neptune_forward_expected)
        self.assertDictEqual(actual_serialized[1], neptune_reversed_expected)

    def test_dashboard_owner_record(self) -> None:

        actual = self.dashboard_owner.create_next_record()
        actual_serialized = mysql_serializer.serialize_record(actual)
        expected = {
            'user_rk': 'foo@bar.com',
            'dashboard_rk': 'product_id_dashboard://cluster_id.dashboard_group_id/dashboard_id'
        }

        assert actual is not None
        self.assertDictEqual(expected, actual_serialized)
        self.assertIsNone(self.dashboard_owner.create_next_record())



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import json
import shutil
import tempfile
import unittest
from typing import List

from pyhocon import ConfigFactory

from databuilder import Scoped
from databuilder.loader.file_system_elasticsearch_json_loader import FSElasticsearchJSONLoader
from databuilder.models.table_elasticsearch_document import TableESDocument


class TestFSElasticsearchJSONLoader(unittest.TestCase):

    def setUp(self) -> None:
        self.temp_dir_path = tempfile.mkdtemp()
        self.dest_file_name = f'{self.temp_dir_path}/test_file.json'
        self.file_mode = 'w'
        config_dict = {'loader.filesystem.elasticsearch.file_path': self.dest_file_name,
                       'loader.filesystem.elasticsearch.mode': self.file_mode}
        self.conf = ConfigFactory.from_dict(config_dict)

    def tearDown(self) -> None:
        shutil.rmtree(self.temp_dir_path)

    def _check_results_helper(self, expected: List[str]) -> None:
        """
        Helper function to compare results with expected outcome
        :param expected: expected result
        """
        with open(self.dest_file_name, 'r') as file:
            for e in expected:
                actual = file.readline().rstrip('\r\n')
                self.assertDictEqual(json.loads(e), json.loads(actual))
            self.assertFalse(file.readline())

    def test_empty_loading(self) -> None:
        """
        Test loading functionality with no data
        """
        loader = FSElasticsearchJSONLoader()
        loader.init(conf=Scoped.get_scoped_conf(conf=self.conf,
                                                scope=loader.get_scope()))

        loader.load(None)  # type: ignore
        loader.close()

        self._check_results_helper(expected=[])

    def test_loading_with_different_object(self) -> None:
        """
        Test Loading functionality with a python Dict object
        """
        loader = FSElasticsearchJSONLoader()
        loader.init(conf=Scoped.get_scoped_conf(conf=self.conf,
                                                scope=loader.get_scope()))

        data = dict(database='test_database',
                    cluster='test_cluster',
                    schema='test_schema',
                    name='test_table',
                    key='test_table_key',
                    last_updated_timestamp=123456789,
                    description='test_description',
                    column_names=['test_col1', 'test_col2'],
                    column_descriptions=['test_comment1', 'test_comment2'],
                    total_usage=10,
                    unique_usage=5,
                    tags=['test_tag1', 'test_tag2'],
                    programmatic_descriptions=['test'])

        with self.assertRaises(Exception) as context:
            loader.load(data)  # type: ignore
        self.assertIn("Record not of type 'ElasticsearchDocument'!", str(context.exception))

        loader.close()

    def test_loading_with_single_object(self) -> None:
        """
        Test Loading functionality with single python object
        """
        loader = FSElasticsearchJSONLoader()
        loader.init(conf=Scoped.get_scoped_conf(conf=self.conf,
                                                scope=loader.get_scope()))

        data = TableESDocument(database='test_database',
                               cluster='test_cluster',
                               schema='test_schema',
                               name='test_table',
                               key='test_table_key',
                               last_updated_timestamp=123456789,
                               description='test_description',
                               column_names=['test_col1', 'test_col2'],
                               column_descriptions=['test_comment1', 'test_comment2'],
                               total_usage=10,
                               unique_usage=5,
                               tags=['test_tag1', 'test_tag2'],
                               badges=['badge1'],
                               schema_description='schema description',
                               programmatic_descriptions=['test'])
        loader.load(data)
        loader.close()

        expected = [
            ('{"key": "test_table_key", "column_descriptions": ["test_comment1", "test_comment2"], '
             '"schema": "test_schema", "database": "test_database", "cluster": "test_cluster", '
             '"column_names": ["test_col1", "test_col2"], "name": "test_table", '
             '"last_updated_timestamp": 123456789, "display_name": "test_schema.test_table", '
             '"description": "test_description", "unique_usage": 5, "total_usage": 10, '
             '"tags": ["test_tag1", "test_tag2"], "schema_description": "schema description", '
             '"programmatic_descriptions": ["test"], '
             '"badges": ["badge1"]}')
        ]

        self._check_results_helper(expected=expected)

    def test_loading_with_list_of_objects(self) -> None:
        """
        Test Loading functionality with list of objects.
        Check to ensure all objects are added to file
        """
        loader = FSElasticsearchJSONLoader()
        loader.init(conf=Scoped.get_scoped_conf(conf=self.conf,
                                                scope=loader.get_scope()))

        data = [TableESDocument(database='test_database',
                                cluster='test_cluster',
                                schema='test_schema',
                                name='test_table',
                                key='test_table_key',
                                last_updated_timestamp=123456789,
                                description='test_description',
                                column_names=['test_col1', 'test_col2'],
                                column_descriptions=['test_comment1', 'test_comment2'],
                                total_usage=10,
                                unique_usage=5,
                                tags=['test_tag1', 'test_tag2'],
                                badges=['badge1'],
                                schema_description='schema_description',
                                programmatic_descriptions=['test'])] * 5

        for d in data:
            loader.load(d)
        loader.close()

        expected = [
            ('{"key": "test_table_key", "column_descriptions": ["test_comment1", "test_comment2"], '
             '"schema": "test_schema", "database": "test_database", "cluster": "test_cluster", '
             '"column_names": ["test_col1", "test_col2"], "name": "test_table", '
             '"last_updated_timestamp": 123456789, "display_name": "test_schema.test_table", '
             '"description": "test_description", "unique_usage": 5, "total_usage": 10, '
             '"tags": ["test_tag1", "test_tag2"], "schema_description": "schema_description", '
             '"programmatic_descriptions":["test"], '
             '"badges": ["badge1"]}')
        ] * 5

        self._check_results_helper(expected=expected)



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import unittest
from typing import Any, Dict
from unittest.mock import ANY

from databuilder.models.dashboard.dashboard_last_modified import DashboardLastModifiedTimestamp
from databuilder.models.graph_serializable import (
    RELATION_END_KEY, RELATION_END_LABEL, RELATION_REVERSE_TYPE, RELATION_START_KEY, RELATION_START_LABEL,
    RELATION_TYPE,
)
from databuilder.serializers import (
    mysql_serializer, neo4_serializer, neptune_serializer,
)
from databuilder.serializers.neptune_serializer import (
    METADATA_KEY_PROPERTY_NAME, NEPTUNE_CREATION_TYPE_JOB, NEPTUNE_CREATION_TYPE_NODE_PROPERTY_NAME_BULK_LOADER_FORMAT,
    NEPTUNE_CREATION_TYPE_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT, NEPTUNE_HEADER_ID, NEPTUNE_HEADER_LABEL,
    NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT, NEPTUNE_RELATIONSHIP_HEADER_FROM,
    NEPTUNE_RELATIONSHIP_HEADER_TO,
)


class TestDashboardLastModifiedTimestamp(unittest.TestCase):

    def setUp(self) -> None:
        self.dashboard_last_modified = DashboardLastModifiedTimestamp(
            last_modified_timestamp=123456789,
            cluster='cluster_id',
            product='product_id',
            dashboard_id='dashboard_id',
            dashboard_group_id='dashboard_group_id'
        )

        self.expected_ts_key = 'product_id_dashboard://cluster_id.dashboard_group_id/dashboard_id/' \
                               '_last_modified_timestamp'
        self.expected_dashboard_key = 'product_id_dashboard://cluster_id.dashboard_group_id/dashboard_id'

    def test_dashboard_timestamp_nodes(self) -> None:

        actual = self.dashboard_last_modified.create_next_node()
        actual_serialized = neo4_serializer.serialize_node(actual)

        expected: Dict[str, Any] = {
            'timestamp:UNQUOTED': 123456789,
            'name': 'last_updated_timestamp',
            'KEY': self.expected_ts_key,
            'LABEL': 'Timestamp'
        }

        assert actual is not None
        self.assertDictEqual(actual_serialized, expected)

        self.assertIsNone(self.dashboard_last_modified.create_next_node())

    def test_neptune_dashboard_timestamp_nodes(self) -> None:
        actual = self.dashboard_last_modified.create_next_node()
        actual_neptune_serialized = neptune_serializer.convert_node(actual)
        neptune_expected = {
            NEPTUNE_HEADER_ID: 'Timestamp:' + self.expected_ts_key,
            METADATA_KEY_PROPERTY_NAME: 'Timestamp:' + self.expected_ts_key,
            NEPTUNE_HEADER_LABEL: 'Timestamp',
            NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
            NEPTUNE_CREATION_TYPE_NODE_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB,
            'name:String(single)': 'last_updated_timestamp',
            'timestamp:Long(single)': 123456789,
        }

        self.assertDictEqual(actual_neptune_serialized, neptune_expected)

    def test_dashboard_owner_relations(self) -> None:

        actual = self.dashboard_last_modified.create_next_relation()
        actual_serialized = neo4_serializer.serialize_relationship(actual)

        expected: Dict[str, Any] = {
            RELATION_END_KEY: self.expected_ts_key,
            RELATION_START_LABEL: 'Dashboard',
            RELATION_END_LABEL: 'Timestamp',
            RELATION_START_KEY: self.expected_dashboard_key,
            RELATION_TYPE: 'LAST_UPDATED_AT',
            RELATION_REVERSE_TYPE: 'LAST_UPDATED_TIME_OF'
        }

        assert actual is not None
        self.assertDictEqual(actual_serialized, expected)
        self.assertIsNone(self.dashboard_last_modified.create_next_relation())

    def test_dashboard_owner_relations_neptune(self) -> None:
        actual = self.dashboard_last_modified.create_next_relation()
        actual_serialized = neptune_serializer.convert_relationship(actual)
        neptune_forward_expected = {
            NEPTUNE_HEADER_ID: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Dashboard:' + self.expected_dashboard_key,
                to_vertex_id='Timestamp:' + self.expected_ts_key,
                label='LAST_UPDATED_AT'
            ),
            METADATA_KEY_PROPERTY_NAME: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Dashboard:' + self.expected_dashboard_key,
                to_vertex_id='Timestamp:' + self.expected_ts_key,
                label='LAST_UPDATED_AT'
            ),
            NEPTUNE_RELATIONSHIP_HEADER_FROM: 'Dashboard:' + self.expected_dashboard_key,
            NEPTUNE_RELATIONSHIP_HEADER_TO: 'Timestamp:' + self.expected_ts_key,
            NEPTUNE_HEADER_LABEL: 'LAST_UPDATED_AT',
            NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
            NEPTUNE_CREATION_TYPE_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB
        }

        neptune_reversed_expected = {
            NEPTUNE_HEADER_ID: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Timestamp:' + self.expected_ts_key,
                to_vertex_id='Dashboard:' + self.expected_dashboard_key,
                label='LAST_UPDATED_TIME_OF'
            ),
            METADATA_KEY_PROPERTY_NAME: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Timestamp:' + self.expected_ts_key,
                to_vertex_id='Dashboard:' + self.expected_dashboard_key,
                label='LAST_UPDATED_TIME_OF'
            ),
            NEPTUNE_RELATIONSHIP_HEADER_FROM: 'Timestamp:' + self.expected_ts_key,
            NEPTUNE_RELATIONSHIP_HEADER_TO: 'Dashboard:' + self.expected_dashboard_key,
            NEPTUNE_HEADER_LABEL: 'LAST_UPDATED_TIME_OF',
            NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
            NEPTUNE_CREATION_TYPE_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB
        }
        self.maxDiff = None
        assert actual is not None
        self.assertDictEqual(actual_serialized[0], neptune_forward_expected)
        self.assertDictEqual(actual_serialized[1], neptune_reversed_expected)
        self.assertIsNone(self.dashboard_last_modified.create_next_relation())

    def test_dashboard_timestamp_records(self) -> None:

        actual = self.dashboard_last_modified.create_next_record()
        actual_serialized = mysql_serializer.serialize_record(actual)

        expected = {
            'rk': 'product_id_dashboard://cluster_id.dashboard_group_id/dashboard_id/_last_modified_timestamp',
            'timestamp': 123456789,
            'name': 'last_updated_timestamp',
            'dashboard_rk': 'product_id_dashboard://cluster_id.dashboard_group_id/dashboard_id'
        }

        assert actual is not None
        self.assertDictEqual(actual_serialized, expected)
        self.assertIsNone(self.dashboard_last_modified.create_next_record())



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import unittest

from mock import patch
from pyhocon import ConfigFactory

from databuilder import Scoped
from databuilder.extractor.dashboard.mode_analytics.batch.mode_dashboard_charts_batch_extractor import (
    ModeDashboardChartsBatchExtractor,
)


class TestModeDashboardChartsBatchExtractor(unittest.TestCase):
    def setUp(self) -> None:
        config = ConfigFactory.from_dict({
            'extractor.mode_dashboard_chart_batch.organization': 'amundsen',
            'extractor.mode_dashboard_chart_batch.mode_user_token': 'amundsen_user_token',
            'extractor.mode_dashboard_chart_batch.mode_password_token': 'amundsen_password_token',
            'extractor.mode_dashboard_chart_batch.mode_bearer_token': 'amundsen_bearer_token',
        })
        self.config = config

    def test_dashboard_chart_extractor_empty_record(self) -> None:
        extractor = ModeDashboardChartsBatchExtractor()
        extractor.init(Scoped.get_scoped_conf(conf=self.config, scope=extractor.get_scope()))

        with patch('databuilder.rest_api.rest_api_query.requests.get'):
            record = extractor.extract()
            self.assertIsNone(record)

    def test_dashboard_chart_extractor_actual_record(self) -> None:
        extractor = ModeDashboardChartsBatchExtractor()
        extractor.init(Scoped.get_scoped_conf(conf=self.config, scope=extractor.get_scope()))

        with patch('databuilder.extractor.restapi.rest_api_extractor.RestAPIExtractor.extract') as mock_get:
            mock_get.return_value = {
                'organization': 'amundsen',
                'is_active': None,
                'updated_at': None,
                'do_not_update_empty_attribute': True,
                'dashboard_group_id': 'ggg',
                'dashboard_id': 'ddd',
                'query_id': 'yyy',
                'chart_id': 'xxx',
                'chart_name': 'some chart',
                'chart_type': 'bigNumber',
                'product': 'mode'
            }

            record = extractor.extract()
            self.assertEqual(record._dashboard_group_id, 'ggg')
            self.assertEqual(record._dashboard_id, 'ddd')
            self.assertEqual(record._chart_name, 'some chart')
            self.assertEqual(record._product, 'mode')


if __name__ == '__main__':
    unittest.main()



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import copy
import unittest
from typing import Dict, List

from databuilder.models.table_metadata import ColumnMetadata, TableMetadata
from databuilder.serializers import (
    mysql_serializer, neo4_serializer, neptune_serializer,
)
from tests.unit.models.test_fixtures.table_metadata_fixtures import (
    EXPECTED_NEPTUNE_NODES, EXPECTED_RECORDS_MYSQL, EXPECTED_RELATIONSHIPS_NEPTUNE,
)


class TestTableMetadata(unittest.TestCase):
    def setUp(self) -> None:
        super(TestTableMetadata, self).setUp()
        TableMetadata.serialized_nodes_keys = set()
        TableMetadata.serialized_rels_keys = set()
        self.table_metadata = TableMetadata(
            'hive',
            'gold',
            'test_schema1',
            'test_table1',
            'test_table1',
            [
                ColumnMetadata('test_id1', 'description of test_table1', 'bigint', 0),
                ColumnMetadata('test_id2', 'description of test_id2', 'bigint', 1),
                ColumnMetadata('is_active', None, 'boolean', 2),
                ColumnMetadata('source', 'description of source', 'varchar', 3),
                ColumnMetadata('etl_created_at', 'description of etl_created_at', 'timestamp', 4),
                ColumnMetadata('ds', None, 'varchar', 5)
            ]
        )

        self.table_metadata2 = TableMetadata(
            'hive',
            'gold',
            'test_schema1',
            'test_table1',
            'test_table1',
            [
                ColumnMetadata('test_id1', 'description of test_table1', 'bigint', 0),
                ColumnMetadata('test_id2', 'description of test_id2', 'bigint', 1),
                ColumnMetadata('is_active', None, 'boolean', 2),
                ColumnMetadata('source', 'description of source', 'varchar', 3),
                ColumnMetadata('etl_created_at', 'description of etl_created_at', 'timestamp', 4),
                ColumnMetadata('ds', None, 'varchar', 5)
            ]
        )

    def test_serialize(self) -> None:
        self.expected_nodes_deduped = [
            {'name': 'test_table1', 'KEY': 'hive://gold.test_schema1/test_table1', 'LABEL': 'Table',
             'is_view:UNQUOTED': False},
            {'description': 'test_table1', 'KEY': 'hive://gold.test_schema1/test_table1/_description',
             'LABEL': 'Description', 'description_source': 'description'},
            {'sort_order:UNQUOTED': 0, 'col_type': 'bigint', 'name': 'test_id1',
             'KEY': 'hive://gold.test_schema1/test_table1/test_id1', 'LABEL': 'Column'},
            {'description': 'description of test_table1',
             'KEY': 'hive://gold.test_schema1/test_table1/test_id1/_description', 'LABEL': 'Description',
             'description_source': 'description'},
            {'sort_order:UNQUOTED': 1, 'col_type': 'bigint', 'name': 'test_id2',
             'KEY': 'hive://gold.test_schema1/test_table1/test_id2', 'LABEL': 'Column'},
            {'description': 'description of test_id2',
             'KEY': 'hive://gold.test_schema1/test_table1/test_id2/_description',
             'LABEL': 'Description', 'description_source': 'description'},
            {'sort_order:UNQUOTED': 2, 'col_type': 'boolean', 'name': 'is_active',
             'KEY': 'hive://gold.test_schema1/test_table1/is_active', 'LABEL': 'Column'},
            {'sort_order:UNQUOTED': 3, 'col_type': 'varchar', 'name': 'source',
             'KEY': 'hive://gold.test_schema1/test_table1/source', 'LABEL': 'Column'},
            {'description': 'description of source', 'KEY': 'hive://gold.test_schema1/test_table1/source/_description',
             'LABEL': 'Description', 'description_source': 'description'},
            {'sort_order:UNQUOTED': 4, 'col_type': 'timestamp', 'name': 'etl_created_at',
             'KEY': 'hive://gold.test_schema1/test_table1/etl_created_at', 'LABEL': 'Column'},
            {'description': 'description of etl_created_at',
             'KEY': 'hive://gold.test_schema1/test_table1/etl_created_at/_description', 'LABEL': 'Description',
             'description_source': 'description'},
            {'sort_order:UNQUOTED': 5, 'col_type': 'varchar', 'name': 'ds',
             'KEY': 'hive://gold.test_schema1/test_table1/ds', 'LABEL': 'Column'}
        ]

        self.expected_nodes = copy.deepcopy(self.expected_nodes_deduped)
        self.expected_nodes.append({'name': 'hive', 'KEY': 'database://hive', 'LABEL': 'Database'})
        self.expected_nodes.append({'name': 'gold', 'KEY': 'hive://gold', 'LABEL': 'Cluster'})
        self.expected_nodes.append({'name': 'test_schema1', 'KEY': 'hive://gold.test_schema1', 'LABEL': 'Schema'})

        self.expected_rels_deduped = [
            {'END_KEY': 'hive://gold.test_schema1/test_table1', 'START_LABEL': 'Schema', 'END_LABEL': 'Table',
             'START_KEY': 'hive://gold.test_schema1', 'TYPE': 'TABLE', 'REVERSE_TYPE': 'TABLE_OF'},
            {'END_KEY': 'hive://gold.test_schema1/test_table1/_description', 'START_LABEL': 'Table',
             'END_LABEL': 'Description', 'START_KEY': 'hive://gold.test_schema1/test_table1', 'TYPE': 'DESCRIPTION',
             'REVERSE_TYPE': 'DESCRIPTION_OF'},
            {'END_KEY': 'hive://gold.test_schema1/test_table1/test_id1', 'START_LABEL': 'Table',
             'END_LABEL': 'Column', 'START_KEY': 'hive://gold.test_schema1/test_table1', 'TYPE': 'COLUMN',
             'REVERSE_TYPE': 'COLUMN_OF'},
            {'END_KEY': 'hive://gold.test_schema1/test_table1/test_id1/_description', 'START_LABEL': 'Column',
             'END_LABEL': 'Description', 'START_KEY': 'hive://gold.test_schema1/test_table1/test_id1',
             'TYPE': 'DESCRIPTION', 'REVERSE_TYPE': 'DESCRIPTION_OF'},
            {'END_KEY': 'hive://gold.test_schema1/test_table1/test_id2', 'START_LABEL': 'Table', 'END_LABEL': 'Column',
             'START_KEY': 'hive://gold.test_schema1/test_table1', 'TYPE': 'COLUMN', 'REVERSE_TYPE': 'COLUMN_OF'},
            {'END_KEY': 'hive://gold.test_schema1/test_table1/test_id2/_description', 'START_LABEL': 'Column',
             'END_LABEL': 'Description', 'START_KEY': 'hive://gold.test_schema1/test_table1/test_id2',
             'TYPE': 'DESCRIPTION', 'REVERSE_TYPE': 'DESCRIPTION_OF'},
            {'END_KEY': 'hive://gold.test_schema1/test_table1/is_active', 'START_LABEL': 'Table', 'END_LABEL': 'Column',
             'START_KEY': 'hive://gold.test_schema1/test_table1', 'TYPE': 'COLUMN', 'REVERSE_TYPE': 'COLUMN_OF'},
            {'END_KEY': 'hive://gold.test_schema1/test_table1/source', 'START_LABEL': 'Table', 'END_LABEL': 'Column',
             'START_KEY': 'hive://gold.test_schema1/test_table1', 'TYPE': 'COLUMN', 'REVERSE_TYPE': 'COLUMN_OF'},
            {'END_KEY': 'hive://gold.test_schema1/test_table1/source/_description', 'START_LABEL': 'Column',
             'END_LABEL': 'Description', 'START_KEY': 'hive://gold.test_schema1/test_table1/source',
             'TYPE': 'DESCRIPTION',
             'REVERSE_TYPE': 'DESCRIPTION_OF'},
            {'END_KEY': 'hive://gold.test_schema1/test_table1/etl_created_at', 'START_LABEL': 'Table',
             'END_LABEL': 'Column', 'START_KEY': 'hive://gold.test_schema1/test_table1', 'TYPE': 'COLUMN',
             'REVERSE_TYPE': 'COLUMN_OF'},
            {'END_KEY': 'hive://gold.test_schema1/test_table1/etl_created_at/_description', 'START_LABEL': 'Column',
             'END_LABEL': 'Description', 'START_KEY': 'hive://gold.test_schema1/test_table1/etl_created_at',
             'TYPE': 'DESCRIPTION', 'REVERSE_TYPE': 'DESCRIPTION_OF'},
            {'END_KEY': 'hive://gold.test_schema1/test_table1/ds', 'START_LABEL': 'Table', 'END_LABEL': 'Column',
             'START_KEY': 'hive://gold.test_schema1/test_table1', 'TYPE': 'COLUMN', 'REVERSE_TYPE': 'COLUMN_OF'}
        ]

        self.expected_rels = copy.deepcopy(self.expected_rels_deduped)
        self.expected_rels.append({'END_KEY': 'hive://gold', 'START_LABEL': 'Database', 'END_LABEL': 'Cluster',
                                   'START_KEY': 'database://hive', 'TYPE': 'CLUSTER', 'REVERSE_TYPE': 'CLUSTER_OF'})
        self.expected_rels.append({'END_KEY': 'hive://gold.test_schema1', 'START_LABEL': 'Cluster',
                                   'END_LABEL': 'Schema', 'START_KEY': 'hive://gold',
                                   'TYPE': 'SCHEMA', 'REVERSE_TYPE': 'SCHEMA_OF'})

        node_row = self.table_metadata.next_node()
        actual = []
        while node_row:
            node_row_serialized = neo4_serializer.serialize_node(node_row)
            actual.append(node_row_serialized)
            node_row = self.table_metadata.next_node()
        for i in range(0, len(self.expected_nodes)):
            self.assertEqual(actual[i], self.expected_nodes[i])

        relation_row = self.table_metadata.next_relation()
        actual = []
        while relation_row:
            relation_row_serialized = neo4_serializer.serialize_relationship(relation_row)
            actual.append(relation_row_serialized)
            relation_row = self.table_metadata.next_relation()
        for i in range(0, len(self.expected_rels)):
            self.assertEqual(actual[i], self.expected_rels[i])

        # 2nd record should not show already serialized database, cluster, and schema
        node_row = self.table_metadata2.next_node()
        actual = []
        while node_row:
            node_row_serialized = neo4_serializer.serialize_node(node_row)
            actual.append(node_row_serialized)
            node_row = self.table_metadata2.next_node()

        self.assertEqual(self.expected_nodes_deduped, actual)

        relation_row = self.table_metadata2.next_relation()
        actual = []
        while relation_row:
            relation_row_serialized = neo4_serializer.serialize_relationship(relation_row)
            actual.append(relation_row_serialized)
            relation_row = self.table_metadata2.next_relation()

        self.assertEqual(self.expected_rels_deduped, actual)

    def test_serialize_neptune(self) -> None:
        node_row = self.table_metadata.next_node()
        actual = []
        while node_row:
            node_row_serialized = neptune_serializer.convert_node(node_row)
            actual.append(node_row_serialized)
            node_row = self.table_metadata.next_node()

        self.assertEqual(EXPECTED_NEPTUNE_NODES, actual)

        relation_row = self.table_metadata.next_relation()
        neptune_actual: List[List[Dict]] = []
        while relation_row:
            relation_row_serialized = neptune_serializer.convert_relationship(relation_row)
            neptune_actual.append(relation_row_serialized)
            relation_row = self.table_metadata.next_relation()
        self.maxDiff = None
        self.assertEqual(EXPECTED_RELATIONSHIPS_NEPTUNE, neptune_actual)

    def test_serialize_mysql(self) -> None:
        actual = []
        record = self.table_metadata.next_record()
        while record:
            serialized_record = mysql_serializer.serialize_record(record)
            actual.append(serialized_record)
            record = self.table_metadata.next_record()

        self.assertEqual(EXPECTED_RECORDS_MYSQL, actual)

    def test_table_attributes(self) -> None:
        self.table_metadata3 = TableMetadata('hive', 'gold', 'test_schema3', 'test_table3', 'test_table3', [
            ColumnMetadata('test_id1', 'description of test_table1', 'bigint', 0),
            ColumnMetadata('test_id2', 'description of test_id2', 'bigint', 1),
            ColumnMetadata('is_active', None, 'boolean', 2),
            ColumnMetadata('source', 'description of source', 'varchar', 3),
            ColumnMetadata('etl_created_at', 'description of etl_created_at', 'timestamp', 4),
            ColumnMetadata('ds', None, 'varchar', 5)], is_view=False, attr1='uri', attr2='attr2')

        node_row = self.table_metadata3.next_node()
        actual = []
        while node_row:
            node_row_serialized = neo4_serializer.serialize_node(node_row)
            actual.append(node_row_serialized)
            node_row = self.table_metadata3.next_node()

        self.assertEqual(actual[0].get('attr1'), 'uri')
        self.assertEqual(actual[0].get('attr2'), 'attr2')

    # TODO NO test can run before serialiable... need to fix
    def test_z_custom_sources(self) -> None:
        self.custom_source = TableMetadata('hive', 'gold', 'test_schema3', 'test_table4', 'test_table4', [
            ColumnMetadata('test_id1', 'description of test_table1', 'bigint', 0),
            ColumnMetadata('test_id2', 'description of test_id2', 'bigint', 1),
            ColumnMetadata('is_active', None, 'boolean', 2),
            ColumnMetadata('source', 'description of source', 'varchar', 3),
            ColumnMetadata('etl_created_at', 'description of etl_created_at', 'timestamp', 4),
            ColumnMetadata('ds', None, 'varchar', 5)], is_view=False, description_source="custom")

        node_row = self.custom_source.next_node()
        actual = []
        while node_row:
            node_row_serialized = neo4_serializer.serialize_node(node_row)
            actual.append(node_row_serialized)
            node_row = self.custom_source.next_node()
        expected = {'LABEL': 'Programmatic_Description',
                    'KEY': 'hive://gold.test_schema3/test_table4/_custom_description',
                    'description_source': 'custom', 'description': 'test_table4'}
        self.assertEqual(actual[1], expected)

    def test_tags_field(self) -> None:
        self.table_metadata4 = TableMetadata('hive', 'gold', 'test_schema4', 'test_table4', 'test_table4', [
            ColumnMetadata('test_id1', 'description of test_table1', 'bigint', 0)],
            is_view=False, tags=['tag1', 'tag2'], attr1='uri', attr2='attr2')

        node_row = self.table_metadata4.next_node()
        actual = []
        while node_row:
            node_row_serialized = neo4_serializer.serialize_node(node_row)
            actual.append(node_row_serialized)
            node_row = self.table_metadata4.next_node()

        self.assertEqual(actual[0].get('attr1'), 'uri')
        self.assertEqual(actual[0].get('attr2'), 'attr2')

        self.assertEqual(actual[2].get('LABEL'), 'Tag')
        self.assertEqual(actual[2].get('KEY'), 'tag1')
        self.assertEqual(actual[3].get('KEY'), 'tag2')

        relation_row = self.table_metadata4.next_relation()
        actual = []
        while relation_row:
            relation_row_serialized = neo4_serializer.serialize_relationship(relation_row)
            actual.append(relation_row_serialized)
            relation_row = self.table_metadata4.next_relation()

        # Table tag relationship
        expected_tab_tag_rel1 = {'END_KEY': 'tag1', 'START_LABEL': 'Table', 'END_LABEL':
                                 'Tag', 'START_KEY': 'hive://gold.test_schema4/test_table4',
                                 'TYPE': 'TAGGED_BY', 'REVERSE_TYPE': 'TAG'}
        expected_tab_tag_rel2 = {'END_KEY': 'tag2', 'START_LABEL': 'Table',
                                 'END_LABEL': 'Tag', 'START_KEY': 'hive://gold.test_schema4/test_table4',
                                 'TYPE': 'TAGGED_BY', 'REVERSE_TYPE': 'TAG'}

        self.assertEqual(actual[2], expected_tab_tag_rel1)
        self.assertEqual(actual[3], expected_tab_tag_rel2)

    def test_col_badge_field(self) -> None:
        self.table_metadata4 = TableMetadata('hive', 'gold', 'test_schema4', 'test_table4', 'test_table4', [
            ColumnMetadata('test_id1', 'description of test_table1', 'bigint', 0, ['col-badge1', 'col-badge2'])],
            is_view=False, attr1='uri', attr2='attr2')

        node_row = self.table_metadata4.next_node()
        actual = []
        while node_row:
            serialized_node_row = neo4_serializer.serialize_node(node_row)
            actual.append(serialized_node_row)
            node_row = self.table_metadata4.next_node()

        self.assertEqual(actual[4].get('KEY'), 'col-badge1')
        self.assertEqual(actual[5].get('KEY'), 'col-badge2')

        relation_row = self.table_metadata4.next_relation()
        actual = []
        while relation_row:
            serialized_relation_row = neo4_serializer.serialize_relationship(relation_row)
            actual.append(serialized_relation_row)
            relation_row = self.table_metadata4.next_relation()

        expected_col_badge_rel1 = {'END_KEY': 'col-badge1', 'START_LABEL': 'Column',
                                   'END_LABEL': 'Badge',
                                   'START_KEY': 'hive://gold.test_schema4/test_table4/test_id1',
                                   'TYPE': 'HAS_BADGE', 'REVERSE_TYPE': 'BADGE_FOR'}
        expected_col_badge_rel2 = {'END_KEY': 'col-badge2', 'START_LABEL': 'Column',
                                   'END_LABEL': 'Badge',
                                   'START_KEY': 'hive://gold.test_schema4/test_table4/test_id1',
                                   'TYPE': 'HAS_BADGE', 'REVERSE_TYPE': 'BADGE_FOR'}

        self.assertEqual(actual[4], expected_col_badge_rel1)
        self.assertEqual(actual[5], expected_col_badge_rel2)

    def test_tags_populated_from_str(self) -> None:
        self.table_metadata5 = TableMetadata('hive', 'gold', 'test_schema5', 'test_table5', 'test_table5', [
            ColumnMetadata('test_id1', 'description of test_table1', 'bigint', 0)], tags="tag3, tag4")

        # Test table tag field populated from str
        node_row = self.table_metadata5.next_node()
        actual = []
        while node_row:
            node_row_serialized = neo4_serializer.serialize_node(node_row)
            actual.append(node_row_serialized)
            node_row = self.table_metadata5.next_node()

        self.assertEqual(actual[2].get('LABEL'), 'Tag')
        self.assertEqual(actual[2].get('KEY'), 'tag3')
        self.assertEqual(actual[3].get('KEY'), 'tag4')

        relation_row = self.table_metadata5.next_relation()
        actual = []
        while relation_row:
            relation_row_serialized = neo4_serializer.serialize_relationship(relation_row)
            actual.append(relation_row_serialized)
            relation_row = self.table_metadata5.next_relation()

        # Table tag relationship
        expected_tab_tag_rel3 = {'END_KEY': 'tag3', 'START_LABEL': 'Table', 'END_LABEL':
                                 'Tag', 'START_KEY': 'hive://gold.test_schema5/test_table5',
                                 'TYPE': 'TAGGED_BY', 'REVERSE_TYPE': 'TAG'}
        expected_tab_tag_rel4 = {'END_KEY': 'tag4', 'START_LABEL': 'Table',
                                 'END_LABEL': 'Tag', 'START_KEY': 'hive://gold.test_schema5/test_table5',
                                 'TYPE': 'TAGGED_BY', 'REVERSE_TYPE': 'TAG'}
        self.assertEqual(actual[2], expected_tab_tag_rel3)
        self.assertEqual(actual[3], expected_tab_tag_rel4)

    def test_tags_arent_populated_from_empty_list_and_str(self) -> None:
        self.table_metadata6 = TableMetadata('hive', 'gold', 'test_schema6', 'test_table6', 'test_table6', [
            ColumnMetadata('test_id1', 'description of test_table1', 'bigint', 0)], tags=[])

        self.table_metadata7 = TableMetadata('hive', 'gold', 'test_schema7', 'test_table7', 'test_table7', [
            ColumnMetadata('test_id1', 'description of test_table1', 'bigint', 0)], tags="")

        # Test table tag fields are not populated from empty List
        node_row = self.table_metadata6.next_node()
        while node_row:
            node_row_serialized = neo4_serializer.serialize_node(node_row)
            self.assertNotEqual(node_row_serialized.get('LABEL'), 'Tag')
            node_row = self.table_metadata6.next_node()

        # Test table tag fields are not populated from empty str
        node_row = self.table_metadata7.next_node()
        while node_row:
            node_row_serialized = neo4_serializer.serialize_node(node_row)
            self.assertNotEqual(node_row_serialized.get('LABEL'), 'Tag')
            node_row = self.table_metadata7.next_node()


if __name__ == '__main__':
    unittest.main()



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import base64
import tempfile
import unittest
from typing import Any

from mock import Mock, patch
from pyhocon import ConfigFactory

from databuilder import Scoped
from databuilder.extractor.bigquery_usage_extractor import BigQueryTableUsageExtractor, TableColumnUsageTuple

CORRECT_DATA = {
    "entries": [{
        "protoPayload": {
            "@type": "type.googleapis.com/google.cloud.audit.AuditLog",
            "status": {},
            "authenticationInfo": {
                "principalEmail": "your-user-here@test.com"
            },
            "serviceName": "bigquery.googleapis.com",
            "methodName": "jobservice.jobcompleted",
            "resourceName": "projects/your-project-here/jobs/bquxjob_758c08d1_16a96889839",
            "serviceData": {
                "@type": "type.googleapis.com/google.cloud.bigquery.logging.v1.AuditData",
                "jobCompletedEvent": {
                    "eventName": "query_job_completed",
                    "job": {
                        "jobName": {
                            "projectId": "your-project-here",
                            "jobId": "bquxjob_758c08d1_16a96889839",
                            "location": "US"
                        },
                        "jobConfiguration": {
                            "query": {
                                "query": "select descript from "
                                         "`bigquery-public-data.austin_incidents.incidents_2008`\n",
                                "destinationTable": {
                                    "projectId": "your-project-here",
                                    "datasetId": "_07147a061ddfd6dcaf246cfc5e858a0ccefa7080",
                                    "tableId": "anon1dd83635c62357091e55a5f76fb62d7deebcfa4c"
                                },
                                "createDisposition": "CREATE_IF_NEEDED",
                                "writeDisposition": "WRITE_TRUNCATE",
                                "defaultDataset": {},
                                "queryPriority": "QUERY_INTERACTIVE",
                                "statementType": "SELECT"
                            }
                        },
                        "jobStatus": {
                            "state": "DONE",
                            "error": {}
                        },
                        "jobStatistics": {
                            "createTime": "2019-05-08T08:22:56.349Z",
                            "startTime": "2019-05-08T08:22:56.660Z",
                            "endTime": "2019-05-08T08:23:00.049Z",
                            "totalProcessedBytes": "3637807",
                            "totalBilledBytes": "10485760",
                            "billingTier": 1,
                            "totalSlotMs": "452",
                            "referencedTables": [
                                {
                                    "projectId": "bigquery-public-data",
                                    "datasetId": "austin_incidents",
                                    "tableId": "incidents_2008"
                                }
                            ],
                            "totalTablesProcessed": 1,
                            "queryOutputRowCount": "179524"
                        }
                    }
                }
            }
        },
        "insertId": "-jyqvjse6lwjz",
        "resource": {
            "type": "bigquery_resource",
            "labels": {
                "project_id": "your-project-here"
            }
        },
        "timestamp": "2019-05-08T08:23:00.061Z",
        "severity": "INFO",
        "logName": "projects/your-project-here/logs/cloudaudit.googleapis.com%2Fdata_access",
        "receiveTimestamp": "2019-05-08T08:23:00.310709609Z"
    }]
}  # noqa

FAILURE = {
    "entries": [{
        "protoPayload": {
            "authenticationInfo": {
                "principalEmail": "your-user-here@test.com"
            },
            "methodName": "jobservice.jobcompleted",
            "serviceData": {
                "jobCompletedEvent": {
                    "job": {
                        "jobStatus": {
                            "state": "DONE",
                            "error": {
                                "code": 11,
                                "message": "Some descriptive error message"
                            }
                        },
                        "jobStatistics": {
                            "createTime": "2019-05-08T08:22:56.349Z",
                            "startTime": "2019-05-08T08:22:56.660Z",
                            "endTime": "2019-05-08T08:23:00.049Z",
                            "totalProcessedBytes": "3637807",
                            "totalBilledBytes": "10485760",
                            "referencedTables": [
                                {
                                    "projectId": "bigquery-public-data",
                                    "datasetId": "austin_incidents",
                                    "tableId": "incidents_2008"
                                }
                            ]
                        }
                    }
                }
            },
        },
    }]
}  # noqa

# An empty dict will be ignored, but putting in nextPageToken causes the test
# to loop infinitely, so we need a bogus key/value to ensure that we will try
# to read entries
NO_ENTRIES = {'key': 'value'}  # noqa

KEYFILE_DATA = """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"""


class MockLoggingClient():
    def __init__(self, data: Any) -> None:
        self.data = data
        self.a = Mock()
        self.a.execute.return_value = self.data
        self.b = Mock()
        self.b.list.return_value = self.a

    def entries(self) -> Any:
        return self.b


# Patch fallback auth method to avoid actually calling google API
@patch('google.auth.default', lambda scopes: ['dummy', 'dummy'])
class TestBigqueryUsageExtractor(unittest.TestCase):

    @patch('databuilder.extractor.base_bigquery_extractor.build')
    def test_basic_extraction(self, mock_build: Any) -> None:
        """
        Test Extraction using mock class
        """
        config_dict = {
            f'extractor.bigquery_table_usage.{BigQueryTableUsageExtractor.PROJECT_ID_KEY}': 'your-project-here',
        }
        conf = ConfigFactory.from_dict(config_dict)

        mock_build.return_value = MockLoggingClient(CORRECT_DATA)
        extractor = BigQueryTableUsageExtractor()
        extractor.init(Scoped.get_scoped_conf(conf=conf,
                                              scope=extractor.get_scope()))
        result = extractor.extract()
        assert result is not None
        self.assertIsInstance(result, tuple)

        (key, value) = result
        self.assertIsInstance(key, TableColumnUsageTuple)
        self.assertIsInstance(value, int)

        self.assertEqual(key.database, 'bigquery')
        self.assertEqual(key.cluster, 'bigquery-public-data')
        self.assertEqual(key.schema, 'austin_incidents')
        self.assertEqual(key.table, 'incidents_2008')
        self.assertEqual(key.email, 'your-user-here@test.com')
        self.assertEqual(value, 1)

    @patch('databuilder.extractor.base_bigquery_extractor.build')
    def test_no_entries(self, mock_build: Any) -> None:
        config_dict = {
            f'extractor.bigquery_table_usage.{BigQueryTableUsageExtractor.PROJECT_ID_KEY}': 'your-project-here',
        }
        conf = ConfigFactory.from_dict(config_dict)

        mock_build.return_value = MockLoggingClient(NO_ENTRIES)
        extractor = BigQueryTableUsageExtractor()
        extractor.init(Scoped.get_scoped_conf(conf=conf,
                                              scope=extractor.get_scope()))
        result = extractor.extract()
        self.assertIsNone(result)

    @patch('databuilder.extractor.base_bigquery_extractor.build')
    def test_key_path(self, mock_build: Any) -> None:
        """
        Test key_path can be used
        """

        with tempfile.NamedTemporaryFile() as keyfile:
            # There are many github scanners looking for API / cloud keys, so in order not to get a
            # false positive triggering everywhere, I base64 encoded the key.
            # This is written to a tempfile as part of this test and then used.
            keyfile.write(base64.b64decode(KEYFILE_DATA))
            keyfile.flush()
            config_dict = {
                f'extractor.bigquery_table_usage.{BigQueryTableUsageExtractor.PROJECT_ID_KEY}': 'your-project-here',
                f'extractor.bigquery_table_usage.{BigQueryTableUsageExtractor.KEY_PATH_KEY}': keyfile.name,
            }
            conf = ConfigFactory.from_dict(config_dict)

            mock_build.return_value = MockLoggingClient(CORRECT_DATA)
            extractor = BigQueryTableUsageExtractor()
            extractor.init(Scoped.get_scoped_conf(conf=conf,
                                                  scope=extractor.get_scope()))

            args, kwargs = mock_build.call_args
            creds = kwargs['http'].credentials
            self.assertEqual(creds.project_id, 'your-project-here')
            self.assertEqual(creds.service_account_email, 'test-162@your-project-here.iam.gserviceaccount.com')

    @patch('databuilder.extractor.base_bigquery_extractor.build')
    def test_timestamp_pagesize_settings(self, mock_build: Any) -> None:
        """
        Test timestamp and pagesize can be set
        """
        TIMESTAMP = '2019-01-01T00:00:00.00Z'
        PAGESIZE = 215

        config_dict = {
            f'extractor.bigquery_table_usage.{BigQueryTableUsageExtractor.PROJECT_ID_KEY}': 'your-project-here',
            f'extractor.bigquery_table_usage.{BigQueryTableUsageExtractor.TIMESTAMP_KEY}': TIMESTAMP,
            f'extractor.bigquery_table_usage.{BigQueryTableUsageExtractor.PAGE_SIZE_KEY}': PAGESIZE,
        }
        conf = ConfigFactory.from_dict(config_dict)

        client = MockLoggingClient(CORRECT_DATA)
        mock_build.return_value = client
        extractor = BigQueryTableUsageExtractor()
        extractor.init(Scoped.get_scoped_conf(conf=conf,
                                              scope=extractor.get_scope()))

        args, kwargs = client.b.list.call_args
        body = kwargs['body']

        self.assertEqual(body['pageSize'], PAGESIZE)
        self.assertEqual(TIMESTAMP in body['filter'], True)

    @patch('databuilder.extractor.base_bigquery_extractor.build')
    def test_failed_jobs_should_not_be_counted(self, mock_build: Any) -> None:
        config_dict = {
            f'extractor.bigquery_table_usage.{BigQueryTableUsageExtractor.PROJECT_ID_KEY}': 'your-project-here',
        }
        conf = ConfigFactory.from_dict(config_dict)

        client = MockLoggingClient(FAILURE)
        mock_build.return_value = client
        extractor = BigQueryTableUsageExtractor()
        extractor.init(Scoped.get_scoped_conf(conf=conf,
                                              scope=extractor.get_scope()))

        result = extractor.extract()
        self.assertIsNone(result)

    @patch('databuilder.extractor.base_bigquery_extractor.build')
    def test_email_filter_not_counted(self, mock_build: Any) -> None:
        config_dict = {
            f'extractor.bigquery_table_usage.{BigQueryTableUsageExtractor.PROJECT_ID_KEY}': 'your-project-here',
            f'extractor.bigquery_table_usage.{BigQueryTableUsageExtractor.EMAIL_PATTERN}': 'emailFilter',
        }
        conf = ConfigFactory.from_dict(config_dict)

        mock_build.return_value = MockLoggingClient(CORRECT_DATA)
        extractor = BigQueryTableUsageExtractor()
        extractor.init(Scoped.get_scoped_conf(conf=conf,
                                              scope=extractor.get_scope()))
        result = extractor.extract()
        self.assertIsNone(result)

    @patch('databuilder.extractor.base_bigquery_extractor.build')
    def test_email_filter_counted(self, mock_build: Any) -> None:
        config_dict = {
            f'extractor.bigquery_table_usage.{BigQueryTableUsageExtractor.PROJECT_ID_KEY}': 'your-project-here',
            f'extractor.bigquery_table_usage.{BigQueryTableUsageExtractor.EMAIL_PATTERN}': '.*@test.com.*',
        }
        conf = ConfigFactory.from_dict(config_dict)

        mock_build.return_value = MockLoggingClient(CORRECT_DATA)
        extractor = BigQueryTableUsageExtractor()
        extractor.init(Scoped.get_scoped_conf(conf=conf,
                                              scope=extractor.get_scope()))
        result = extractor.extract()
        assert result is not None
        self.assertIsInstance(result, tuple)

        (key, value) = result
        self.assertIsInstance(key, TableColumnUsageTuple)
        self.assertIsInstance(value, int)

        self.assertEqual(key.database, 'bigquery')
        self.assertEqual(key.cluster, 'bigquery-public-data')
        self.assertEqual(key.schema, 'austin_incidents')
        self.assertEqual(key.table, 'incidents_2008')
        self.assertEqual(key.email, 'your-user-here@test.com')
        self.assertEqual(value, 1)



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import unittest

from databuilder.models.es_last_updated import ESLastUpdated
from databuilder.models.graph_serializable import NODE_KEY, NODE_LABEL
from databuilder.serializers import mysql_serializer, neo4_serializer


class TestNeo4jESLastUpdated(unittest.TestCase):

    def setUp(self) -> None:
        super(TestNeo4jESLastUpdated, self).setUp()
        self.es_last_updated = ESLastUpdated(timestamp=100)

        self.expected_node_results = [{
            NODE_KEY: 'amundsen_updated_timestamp',
            NODE_LABEL: 'Updatedtimestamp',
            'latest_timestamp:UNQUOTED': 100,
        }]

    def test_create_nodes(self) -> None:
        actual = []
        node = self.es_last_updated.create_next_node()
        while node:
            serialized_node = neo4_serializer.serialize_node(node)
            actual.append(serialized_node)
            node = self.es_last_updated.create_next_node()

        self.assertEqual(actual, self.expected_node_results)

    def test_create_next_relation(self) -> None:
        self.assertIs(self.es_last_updated.create_next_relation(), None)

    def test_create_records(self) -> None:
        expected = [{
            'rk': 'amundsen_updated_timestamp',
            'latest_timestamp': 100
        }]

        actual = []
        record = self.es_last_updated.create_next_record()
        while record:
            serialized_record = mysql_serializer.serialize_record(record)
            actual.append(serialized_record)
            record = self.es_last_updated.create_next_record()

        self.assertEqual(expected, actual)



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import unittest
from unittest.mock import ANY

from databuilder.models.dashboard.dashboard_table import DashboardTable
from databuilder.models.graph_serializable import (
    RELATION_END_KEY, RELATION_END_LABEL, RELATION_REVERSE_TYPE, RELATION_START_KEY, RELATION_START_LABEL,
    RELATION_TYPE,
)
from databuilder.serializers import (
    mysql_serializer, neo4_serializer, neptune_serializer,
)
from databuilder.serializers.neptune_serializer import (
    METADATA_KEY_PROPERTY_NAME, NEPTUNE_CREATION_TYPE_JOB,
    NEPTUNE_CREATION_TYPE_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT, NEPTUNE_HEADER_ID, NEPTUNE_HEADER_LABEL,
    NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT, NEPTUNE_RELATIONSHIP_HEADER_FROM,
    NEPTUNE_RELATIONSHIP_HEADER_TO,
)


class TestDashboardTable(unittest.TestCase):
    def test_dashboard_table_nodes(self) -> None:
        dashboard_table = DashboardTable(table_ids=['hive://gold.schema/table1', 'hive://gold.schema/table2'],
                                         cluster='cluster_id', product='product_id',
                                         dashboard_id='dashboard_id', dashboard_group_id='dashboard_group_id')

        actual = dashboard_table.create_next_node()
        self.assertIsNone(actual)

    def test_dashboard_table_relations(self) -> None:
        dashboard_table = DashboardTable(table_ids=['hive://gold.schema/table1'],
                                         cluster='cluster_id', product='product_id',
                                         dashboard_id='dashboard_id', dashboard_group_id='dashboard_group_id')

        actual = dashboard_table.create_next_relation()
        actual_serialized = neo4_serializer.serialize_relationship(actual)
        actual_neptune_serialized = neptune_serializer.convert_relationship(actual)
        expected = {RELATION_END_KEY: 'hive://gold.schema/table1', RELATION_START_LABEL: 'Dashboard',
                    RELATION_END_LABEL: 'Table',
                    RELATION_START_KEY: 'product_id_dashboard://cluster_id.dashboard_group_id/dashboard_id',
                    RELATION_TYPE: 'DASHBOARD_WITH_TABLE',
                    RELATION_REVERSE_TYPE: 'TABLE_OF_DASHBOARD'}

        neptune_forward_expected = {
            NEPTUNE_HEADER_ID: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Dashboard:product_id_dashboard://cluster_id.dashboard_group_id/dashboard_id',
                to_vertex_id='Table:hive://gold.schema/table1',
                label='DASHBOARD_WITH_TABLE'
            ),
            METADATA_KEY_PROPERTY_NAME: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Dashboard:product_id_dashboard://cluster_id.dashboard_group_id/dashboard_id',
                to_vertex_id='Table:hive://gold.schema/table1',
                label='DASHBOARD_WITH_TABLE'
            ),
            NEPTUNE_RELATIONSHIP_HEADER_FROM:
                'Dashboard:product_id_dashboard://cluster_id.dashboard_group_id/dashboard_id',
            NEPTUNE_RELATIONSHIP_HEADER_TO: 'Table:hive://gold.schema/table1',
            NEPTUNE_HEADER_LABEL: 'DASHBOARD_WITH_TABLE',
            NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
            NEPTUNE_CREATION_TYPE_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB
        }

        neptune_reversed_expected = {
            NEPTUNE_HEADER_ID: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Table:hive://gold.schema/table1',
                to_vertex_id='Dashboard:product_id_dashboard://cluster_id.dashboard_group_id/dashboard_id',
                label='TABLE_OF_DASHBOARD'
            ),
            METADATA_KEY_PROPERTY_NAME: "{label}:{from_vertex_id}_{to_vertex_id}".format(
                from_vertex_id='Table:hive://gold.schema/table1',
                to_vertex_id='Dashboard:product_id_dashboard://cluster_id.dashboard_group_id/dashboard_id',
                label='TABLE_OF_DASHBOARD'
            ),
            NEPTUNE_RELATIONSHIP_HEADER_FROM: 'Table:hive://gold.schema/table1',
            NEPTUNE_RELATIONSHIP_HEADER_TO:
                'Dashboard:product_id_dashboard://cluster_id.dashboard_group_id/dashboard_id',
            NEPTUNE_HEADER_LABEL: 'TABLE_OF_DASHBOARD',
            NEPTUNE_LAST_EXTRACTED_AT_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: ANY,
            NEPTUNE_CREATION_TYPE_RELATIONSHIP_PROPERTY_NAME_BULK_LOADER_FORMAT: NEPTUNE_CREATION_TYPE_JOB
        }
        assert actual is not None
        self.assertDictEqual(actual_serialized, expected)
        self.assertDictEqual(actual_neptune_serialized[0], neptune_forward_expected)
        self.assertDictEqual(actual_neptune_serialized[1], neptune_reversed_expected)

    def test_dashboard_table_without_dot_as_name(self) -> None:
        dashboard_table = DashboardTable(table_ids=['bq-name://project-id.schema-name/table-name'],
                                         cluster='cluster_id', product='product_id',
                                         dashboard_id='dashboard_id', dashboard_group_id='dashboard_group_id')
        actual = dashboard_table.create_next_relation()
        actual_serialized = neo4_serializer.serialize_relationship(actual)
        expected = {RELATION_END_KEY: 'bq-name://project-id.schema-name/table-name', RELATION_START_LABEL: 'Dashboard',
                    RELATION_END_LABEL: 'Table',
                    RELATION_START_KEY: 'product_id_dashboard://cluster_id.dashboard_group_id/dashboard_id',
                    RELATION_TYPE: 'DASHBOARD_WITH_TABLE',
                    RELATION_REVERSE_TYPE: 'TABLE_OF_DASHBOARD'}
        assert actual is not None
        self.assertDictEqual(actual_serialized, expected)

    def test_dashboard_table_with_dot_as_name(self) -> None:
        dashboard_table = DashboardTable(table_ids=['bq-name://project.id.schema-name/table-name'],
                                         cluster='cluster_id', product='product_id',
                                         dashboard_id='dashboard_id', dashboard_group_id='dashboard_group_id')
        actual = dashboard_table.create_next_relation()
        self.assertIsNone(actual)

    def test_dashboard_table_with_slash_as_name(self) -> None:
        dashboard_table = DashboardTable(table_ids=['bq/name://project/id.schema/name/table/name'],
                                         cluster='cluster_id', product='product_id',
                                         dashboard_id='dashboard_id', dashboard_group_id='dashboard_group_id')
        actual = dashboard_table.create_next_relation()
        self.assertIsNone(actual)

    def test_dashboard_table_records(self) -> None:
        dashboard_table = DashboardTable(table_ids=['hive://gold.schema/table1', 'hive://gold.schema/table2'],
                                         cluster='cluster_id', product='product_id',
                                         dashboard_id='dashboard_id', dashboard_group_id='dashboard_group_id')
        actual1 = dashboard_table.create_next_record()
        actual1_serialized = mysql_serializer.serialize_record(actual1)
        expected1 = {
            'dashboard_rk': 'product_id_dashboard://cluster_id.dashboard_group_id/dashboard_id',
            'table_rk': 'hive://gold.schema/table1'
        }

        actual2 = dashboard_table.create_next_record()
        actual2_serialized = mysql_serializer.serialize_record(actual2)
        expected2 = {
            'dashboard_rk': 'product_id_dashboard://cluster_id.dashboard_group_id/dashboard_id',
            'table_rk': 'hive://gold.schema/table2'
        }

        assert actual1 is not None
        self.assertDictEqual(expected1, actual1_serialized)

        assert actual2 is not None
        self.assertDictEqual(expected2, actual2_serialized)
        self.assertIsNone(dashboard_table.create_next_record())



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import logging
import unittest

from mock import patch
from pyhocon import ConfigFactory

from databuilder.extractor.glue_extractor import GlueExtractor
from databuilder.models.table_metadata import ColumnMetadata, TableMetadata


# patch whole class to avoid actually calling for boto3.client during tests
@patch('databuilder.extractor.glue_extractor.boto3.client', lambda x: None)
class TestGlueExtractor(unittest.TestCase):
    def setUp(self) -> None:
        logging.basicConfig(level=logging.INFO)

        self.conf = ConfigFactory.from_dict({})

    def test_extraction_with_empty_query_result(self) -> None:
        """
        Test Extraction with empty result from query
        """
        with patch.object(GlueExtractor, '_search_tables'):
            extractor = GlueExtractor()
            extractor.init(self.conf)

            results = extractor.extract()
            self.assertEqual(results, None)

    def test_extraction_with_single_result(self) -> None:
        with patch.object(GlueExtractor, '_search_tables') as mock_search:
            mock_search.return_value = [
                {
                    'Name': 'test_table',
                    'DatabaseName': 'test_schema',
                    'Description': 'a table for testing',
                    'StorageDescriptor': {
                        'Columns': [
                            {
                                'Name': 'col_id1',
                                'Type': 'bigint',
                                'Comment': 'description of id1'
                            },
                            {
                                'Name': 'col_id2',
                                'Type': 'bigint',
                                'Comment': 'description of id2'
                            },
                            {
                                'Name': 'is_active',
                                'Type': 'boolean'
                            },
                            {
                                'Name': 'source',
                                'Type': 'varchar',
                                'Comment': 'description of source'
                            },
                            {
                                'Name': 'etl_created_at',
                                'Type': 'timestamp',
                                'Comment': 'description of etl_created_at'
                            },
                            {
                                'Name': 'ds',
                                'Type': 'varchar'
                            }
                        ]
                    },
                    'PartitionKeys': [
                        {
                            'Name': 'partition_key1',
                            'Type': 'string',
                            'Comment': 'description of partition_key1'
                        },
                    ],
                    'TableType': 'EXTERNAL_TABLE',
                }
            ]

            extractor = GlueExtractor()
            extractor.init(self.conf)
            actual = extractor.extract()
            expected = TableMetadata('glue', 'gold', 'test_schema', 'test_table', 'a table for testing',
                                     [ColumnMetadata('col_id1', 'description of id1', 'bigint', 0),
                                      ColumnMetadata('col_id2', 'description of id2', 'bigint', 1),
                                      ColumnMetadata('is_active', None, 'boolean', 2),
                                      ColumnMetadata('source', 'description of source', 'varchar', 3),
                                      ColumnMetadata('etl_created_at', 'description of etl_created_at', 'timestamp', 4),
                                      ColumnMetadata('ds', None, 'varchar', 5),
                                      ColumnMetadata('partition_key1', 'description of partition_key1', 'string', 6),
                                      ], False)
            self.assertEqual(expected.__repr__(), actual.__repr__())
            self.assertIsNone(extractor.extract())

    def test_extraction_with_multiple_result(self) -> None:
        with patch.object(GlueExtractor, '_search_tables') as mock_search:
            mock_search.return_value = [
                {
                    'Name': 'test_table1',
                    'DatabaseName': 'test_schema1',
                    'Description': 'test table 1',
                    'StorageDescriptor': {
                        'Columns': [
                            {
                                'Name': 'col_id1',
                                'Type': 'bigint',
                                'Comment': 'description of col_id1'
                            },
                            {
                                'Name': 'col_id2',
                                'Type': 'bigint',
                                'Comment': 'description of col_id2'
                            },
                            {
                                'Name': 'is_active',
                                'Type': 'boolean'
                            },
                            {
                                'Name': 'source',
                                'Type': 'varchar',
                                'Comment': 'description of source'
                            },
                            {
                                'Name': 'etl_created_at',
                                'Type': 'timestamp',
                                'Comment': 'description of etl_created_at'
                            },
                            {
                                'Name': 'ds',
                                'Type': 'varchar'
                            }
                        ]
                    },
                    'PartitionKeys': [
                        {
                            'Name': 'partition_key1',
                            'Type': 'string',
                            'Comment': 'description of partition_key1'
                        },
                    ],
                    'TableType': 'EXTERNAL_TABLE',
                },
                {
                    'Name': 'test_table2',
                    'DatabaseName': 'test_schema1',
                    'Description': 'test table 2',
                    'StorageDescriptor': {
                        'Columns': [
                            {
                                'Name': 'col_name',
                                'Type': 'varchar',
                                'Comment': 'description of col_name'
                            },
                            {
                                'Name': 'col_name2',
                                'Type': 'varchar',
                                'Comment': 'description of col_name2'
                            }
                        ]
                    },
                    'TableType': 'EXTERNAL_TABLE',
                },
                {
                    'Name': 'test_table3',
                    'DatabaseName': 'test_schema2',
                    'StorageDescriptor': {
                        'Columns': [
                            {
                                'Name': 'col_id3',
                                'Type': 'varchar',
                                'Comment': 'description of col_id3'
                            },
                            {
                                'Name': 'col_name3',
                                'Type': 'varchar',
                                'Comment': 'description of col_name3'
                            }
                        ]
                    },
                    'Parameters': {'comment': 'description of test table 3 from comment'},
                    'TableType': 'EXTERNAL_TABLE',
                },
                {
                    'Name': 'test_view1',
                    'DatabaseName': 'test_schema1',
                    'Description': 'test view 1',
                    'StorageDescriptor': {
                        'Columns': [
                            {
                                'Name': 'col_id3',
                                'Type': 'varchar',
                                'Comment': 'description of col_id3'
                            },
                            {
                                'Name': 'col_name3',
                                'Type': 'varchar',
                                'Comment': 'description of col_name3'
                            }
                        ]
                    },
                    'TableType': 'VIRTUAL_VIEW',
                },
            ]

            extractor = GlueExtractor()
            extractor.init(self.conf)

            expected = TableMetadata('glue', 'gold', 'test_schema1', 'test_table1', 'test table 1',
                                     [ColumnMetadata('col_id1', 'description of col_id1', 'bigint', 0),
                                      ColumnMetadata('col_id2', 'description of col_id2', 'bigint', 1),
                                      ColumnMetadata('is_active', None, 'boolean', 2),
                                      ColumnMetadata('source', 'description of source', 'varchar', 3),
                                      ColumnMetadata('etl_created_at', 'description of etl_created_at', 'timestamp', 4),
                                      ColumnMetadata('ds', None, 'varchar', 5),
                                      ColumnMetadata('partition_key1', 'description of partition_key1', 'string', 6),
                                      ], False)
            self.assertEqual(expected.__repr__(), extractor.extract().__repr__())

            expected = TableMetadata('glue', 'gold', 'test_schema1', 'test_table2', 'test table 2',
                                     [ColumnMetadata('col_name', 'description of col_name', 'varchar', 0),
                                      ColumnMetadata('col_name2', 'description of col_name2', 'varchar', 1)], False)
            self.assertEqual(expected.__repr__(), extractor.extract().__repr__())

            expected = TableMetadata('glue', 'gold', 'test_schema2', 'test_table3',
                                     'description of test table 3 from comment',
                                     [ColumnMetadata('col_id3', 'description of col_id3', 'varchar', 0),
                                      ColumnMetadata('col_name3', 'description of col_name3',
                                                     'varchar', 1)], False)
            self.assertEqual(expected.__repr__(), extractor.extract().__repr__())

            expected = TableMetadata('glue', 'gold', 'test_schema1', 'test_view1', 'test view 1',
                                     [ColumnMetadata('col_id3', 'description of col_id3', 'varchar', 0),
                                      ColumnMetadata('col_name3', 'description of col_name3',
                                                     'varchar', 1)], True)
            self.assertEqual(expected.__repr__(), extractor.extract().__repr__())

            self.assertIsNone(extractor.extract())
            self.assertIsNone(extractor.extract())


if __name__ == '__main__':
    unittest.main()



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

from typing import (
    Iterator, List, Optional, Union,
)

from amundsen_rds.models import RDSModel
from amundsen_rds.models.table import TableOwner as RDSTableOwner
from amundsen_rds.models.user import User as RDSUser

from databuilder.models.graph_node import GraphNode
from databuilder.models.graph_relationship import GraphRelationship
from databuilder.models.graph_serializable import GraphSerializable
from databuilder.models.owner_constants import OWNER_OF_OBJECT_RELATION_TYPE, OWNER_RELATION_TYPE
from databuilder.models.table_serializable import TableSerializable
from databuilder.models.user import User


class TableOwner(GraphSerializable, TableSerializable):
    """
    Hive table owner model.
    """
    OWNER_TABLE_RELATION_TYPE = OWNER_OF_OBJECT_RELATION_TYPE
    TABLE_OWNER_RELATION_TYPE = OWNER_RELATION_TYPE

    def __init__(self,
                 db_name: str,
                 schema: str,
                 table_name: str,
                 owners: Union[List, str],
                 cluster: str = 'gold',
                 ) -> None:
        self.db = db_name
        self.schema = schema
        self.table = table_name
        if isinstance(owners, str):
            owners = owners.split(',')
        self.owners = [owner.strip() for owner in owners]

        self.cluster = cluster
        self._node_iter = self._create_node_iterator()
        self._relation_iter = self._create_relation_iterator()
        self._record_iter = self._create_record_iterator()

    def create_next_node(self) -> Optional[GraphNode]:
        # return the string representation of the data
        try:
            return next(self._node_iter)
        except StopIteration:
            return None

    def create_next_relation(self) -> Optional[GraphRelationship]:
        try:
            return next(self._relation_iter)
        except StopIteration:
            return None

    def create_next_record(self) -> Union[RDSModel, None]:
        try:
            return next(self._record_iter)
        except StopIteration:
            return None

    def get_owner_model_key(self, owner: str) -> str:
        return User.USER_NODE_KEY_FORMAT.format(email=owner)

    def get_metadata_model_key(self) -> str:
        return f'{self.db}://{self.cluster}.{self.schema}/{self.table}'

    def _create_node_iterator(self) -> Iterator[GraphNode]:
        """
        Create table owner nodes
        :return:
        """
        for owner in self.owners:
            if owner:
                node = GraphNode(
                    key=self.get_owner_model_key(owner),
                    label=User.USER_NODE_LABEL,
                    attributes={
                        User.USER_NODE_EMAIL: owner
                    }
                )
                yield node

    def _create_relation_iterator(self) -> Iterator[GraphRelationship]:
        """
        Create relation map between owner record with original hive table
        :return:
        """
        for owner in self.owners:
            if owner:
                relationship = GraphRelationship(
                    start_key=self.get_owner_model_key(owner),
                    start_label=User.USER_NODE_LABEL,
                    end_key=self.get_metadata_model_key(),
                    end_label='Table',
                    type=TableOwner.OWNER_TABLE_RELATION_TYPE,
                    reverse_type=TableOwner.TABLE_OWNER_RELATION_TYPE,
                    attributes={}
                )
                yield relationship

    def _create_record_iterator(self) -> Iterator[RDSModel]:
        for owner in self.owners:
            if owner:
                user_record = RDSUser(
                    rk=self.get_owner_model_key(owner),
                    email=owner
                )
                yield user_record

                table_owner_record = RDSTableOwner(
                    table_rk=self.get_metadata_model_key(),
                    user_rk=self.get_owner_model_key(owner)
                )
                yield table_owner_record

    def __repr__(self) -> str:
        return f'TableOwner({self.db!r}, {self.cluster!r}, {self.schema!r}, {self.table!r}, {self.owners!r})'



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import shutil
import tempfile
import unittest
from typing import List

from pyhocon import ConfigFactory

from databuilder import Scoped
from databuilder.loader.file_system_csv_loader import FileSystemCSVLoader
from tests.unit.extractor.test_sql_alchemy_extractor import TableMetadataResult


class TestFileSystemCSVLoader(unittest.TestCase):

    def setUp(self) -> None:
        self.temp_dir_path = tempfile.mkdtemp()
        self.dest_file_name = f'{self.temp_dir_path}/test_file.csv'
        self.file_mode = 'w'
        config_dict = {'loader.filesystem.csv.file_path': self.dest_file_name,
                       'loader.filesystem.csv.mode': self.file_mode}
        self.conf = ConfigFactory.from_dict(config_dict)

    def tearDown(self) -> None:
        shutil.rmtree(self.temp_dir_path)

    def _check_results_helper(self, expected: List[str]) -> None:
        """
        Helper function to compare results with expected outcome
        :param expected: expected result
        """
        with open(self.dest_file_name, 'r') as file:
            for e in expected:
                actual = file.readline().rstrip('\r\n')
                self.assertEqual(set(e.split(',')), set(actual.split(',')))
            self.assertFalse(file.readline())

    def test_empty_loading(self) -> None:
        """
        Test loading functionality with no data
        """
        loader = FileSystemCSVLoader()
        loader.init(conf=Scoped.get_scoped_conf(conf=self.conf,
                                                scope=loader.get_scope()))

        loader.load(None)
        loader.close()

        self._check_results_helper(expected=[])

    def test_loading_with_single_object(self) -> None:
        """
        Test Loading functionality with single python object
        """
        loader = FileSystemCSVLoader()
        loader.init(conf=Scoped.get_scoped_conf(conf=self.conf,
                                                scope=loader.get_scope()))

        data = TableMetadataResult(database='test_database',
                                   schema='test_schema',
                                   name='test_table',
                                   description='test_description',
                                   column_name='test_column_name',
                                   column_type='test_column_type',
                                   column_comment='test_column_comment',
                                   owner='test_owner')
        loader.load(data)
        loader.close()

        expected = [
            ','.join(['database', 'schema', 'name', 'description',
                      'column_name', 'column_type', 'column_comment',
                      'owner']),
            ','.join(['test_database', 'test_schema', 'test_table',
                      'test_description', 'test_column_name',
                      'test_column_type', 'test_column_comment',
                      'test_owner'])
        ]

        self._check_results_helper(expected=expected)

    def test_loading_with_list_of_objects(self) -> None:
        """
        Test Loading functionality with list of objects.
        Check to ensure all objects are added to file
        """
        loader = FileSystemCSVLoader()
        loader.init(conf=Scoped.get_scoped_conf(conf=self.conf,
                                                scope=loader.get_scope()))

        data = [TableMetadataResult(database='test_database',
                                    schema='test_schema',
                                    name='test_table',
                                    description='test_description',
                                    column_name='test_column_name',
                                    column_type='test_column_type',
                                    column_comment='test_column_comment',
                                    owner='test_owner')] * 5

        for d in data:
            loader.load(d)
        loader.close()

        expected = [
            ','.join(['database', 'schema', 'name', 'description',
                      'column_name', 'column_type', 'column_comment',
                      'owner'])
        ]
        expected = expected + [
            ','.join(['test_database', 'test_schema', 'test_table',
                      'test_description', 'test_column_name',
                      'test_column_type', 'test_column_comment', 'test_owner']
                     )
        ] * 5

        self._check_results_helper(expected=expected)



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import unittest
from typing import Any, Dict

from mock import patch
from pyhocon import ConfigFactory

from databuilder import Scoped
from databuilder.extractor.sql_alchemy_extractor import SQLAlchemyExtractor


class TestSqlAlchemyExtractor(unittest.TestCase):

    def setUp(self) -> None:
        config_dict = {
            'extractor.sqlalchemy.conn_string': 'TEST_CONNECTION',
            'extractor.sqlalchemy.extract_sql': 'SELECT 1 FROM TEST_TABLE;'
        }
        self.conf = ConfigFactory.from_dict(config_dict)

    @patch.object(SQLAlchemyExtractor, '_get_connection')
    def test_extraction_with_empty_query_result(self: Any,
                                                mock_method: Any) -> None:
        """
        Test Extraction with empty result from query
        """
        extractor = SQLAlchemyExtractor()
        extractor.results = ['']
        extractor.init(Scoped.get_scoped_conf(conf=self.conf,
                                              scope=extractor.get_scope()))

        results = extractor.extract()
        self.assertEqual(results, '')

    @patch.object(SQLAlchemyExtractor, '_get_connection')
    def test_extraction_with_single_query_result(self: Any,
                                                 mock_method: Any) -> None:
        """
        Test Extraction from single result from query
        """
        extractor = SQLAlchemyExtractor()
        extractor.results = [('test_result')]
        extractor.init(Scoped.get_scoped_conf(conf=self.conf,
                                              scope=extractor.get_scope()))
        results = extractor.extract()
        self.assertEqual(results, 'test_result')

    @patch.object(SQLAlchemyExtractor, '_get_connection')
    def test_extraction_with_multiple_query_result(self: Any,
                                                   mock_method: Any) -> None:
        """
        Test Extraction from list of results from query
        """
        extractor = SQLAlchemyExtractor()
        extractor.results = ['test_result', 'test_result2', 'test_result3']
        extractor.init(Scoped.get_scoped_conf(conf=self.conf,
                                              scope=extractor.get_scope()))
        result = [extractor.extract() for _ in range(3)]

        self.assertEqual(len(result), 3)
        self.assertEqual(result,
                         ['test_result', 'test_result2', 'test_result3'])

    @patch.object(SQLAlchemyExtractor, '_get_connection')
    def test_extraction_with_model_class(self: Any, mock_method: Any) -> None:
        """
        Test Extraction using model class
        """
        config_dict = {
            'extractor.sqlalchemy.conn_string': 'TEST_CONNECTION',
            'extractor.sqlalchemy.extract_sql': 'SELECT 1 FROM TEST_TABLE;',
            'extractor.sqlalchemy.model_class':
                'tests.unit.extractor.test_sql_alchemy_extractor.TableMetadataResult'
        }
        self.conf = ConfigFactory.from_dict(config_dict)

        extractor = SQLAlchemyExtractor()
        extractor.results = [dict(database='test_database',
                                  schema='test_schema',
                                  name='test_table',
                                  description='test_description',
                                  column_name='test_column_name',
                                  column_type='test_column_type',
                                  column_comment='test_column_comment',
                                  owner='test_owner')]

        extractor.init(Scoped.get_scoped_conf(conf=self.conf,
                                              scope=extractor.get_scope()))

        result = extractor.extract()

        self.assertIsInstance(result, TableMetadataResult)
        self.assertEqual(result.name, 'test_table')

    @patch('databuilder.extractor.sql_alchemy_extractor.create_engine')
    def test_get_connection(self: Any, mock_method: Any) -> None:
        """
        Test that configs are passed through correctly to the _get_connection method
        """
        extractor = SQLAlchemyExtractor()
        config_dict: Dict[str, Any] = {
            'extractor.sqlalchemy.conn_string': 'TEST_CONNECTION',
            'extractor.sqlalchemy.extract_sql': 'SELECT 1 FROM TEST_TABLE;'
        }
        conf = ConfigFactory.from_dict(config_dict)
        extractor.init(Scoped.get_scoped_conf(conf=conf,
                                              scope=extractor.get_scope()))
        extractor._get_connection()
        mock_method.assert_called_with('TEST_CONNECTION', connect_args={})

        extractor = SQLAlchemyExtractor()
        config_dict = {
            'extractor.sqlalchemy.conn_string': 'TEST_CONNECTION',
            'extractor.sqlalchemy.extract_sql': 'SELECT 1 FROM TEST_TABLE;',
            'extractor.sqlalchemy.connect_args': {"protocol": "https"},
        }
        conf = ConfigFactory.from_dict(config_dict)
        extractor.init(Scoped.get_scoped_conf(conf=conf,
                                              scope=extractor.get_scope()))
        extractor._get_connection()
        mock_method.assert_called_with('TEST_CONNECTION', connect_args={"protocol": "https"})


class TableMetadataResult:
    """
    Table metadata result model.
    SQL result has one row per column
    """

    def __init__(self,
                 database: str,
                 schema: str,
                 name: str,
                 description: str,
                 column_name: str,
                 column_type: str,
                 column_comment: str,
                 owner: str
                 ) -> None:
        self.database = database
        self.schema = schema
        self.name = name
        self.description = description
        self.column_name = column_name
        self.column_type = column_type
        self.column_comment = column_comment
        self.owner = owner
